﻿
// Create object for document control
IpPbxMobileDocumentHelper = new Ext.ux.DocumentHelper();

// IMPORTANT
// Right density must be set before
// IpPbxMobile Application is initialized.
IpPbxMobileDocumentHelper.setRightDensity();

//-------------------------------------------------------------------------
// APPLICATION INITIALISATION
//-------------------------------------------------------------------------

IpPbxMobile = new Ext.Application(
{
    backButtonListenerAttached: false,
    name: 'IpPbxMobile',

    launch: function ()
    {
        this.launched = true;
        this.mainLaunch();
    },

    mainLaunch: function ()
    {
        if (Ext.is.Phone)
        {
            // Check if sencha and phonegap library is loaded
            if (!device || !this.launched)
            {
                return;
            }

            deviceInfo.name = device.name;
            deviceInfo.platform = device.platform;
            deviceInfo.version = device.version;
        }

        // Ready to go
        IpPbxMobile.controllers.StartupController.start();

        // Add phone events
        document.addEventListener("pause", this.onAppPause, false);
        document.addEventListener("resume", this.onAppResume, false);
    },

    showMainGui: function ()
    {
        this.views.viewport = new this.views.Viewport();
        this.views.homecard = this.views.viewport.getComponent('usercard');
        this.views.contactscard = this.views.viewport.getComponent('contactscard');
        this.views.journalcard = this.views.viewport.getComponent('journalcard');
        this.views.dialpadcard = this.views.viewport.getComponent('dialpadcard');
        this.views.morecard = this.views.viewport.getComponent('morecard');
    },

    addBackbuttonListener: function ()
    {
        if (Ext.is.Phone)
        {
            if (!this.backButtonListenerAttached)
            {
                document.addEventListener("backbutton", this.onBackButton, true);
                this.backButtonListenerAttached = true;
            }
        }
    },

    removeBackbuttonListener: function ()
    {
        if (Ext.is.Phone)
        {
            if (this.backButtonListenerAttached)
            {
                document.removeEventListener("backbutton", this.onBackButton, true);
                this.backButtonListenerAttached = false;
            }
        }
    },
       
    checkBackButtonHandler: function (pageId)
    {
        if (pageId === 'usercard')
        {
            // Remove event listener
            IpPbxMobile.removeBackbuttonListener();
        }
        else
        {
            // Add listener
            IpPbxMobile.addBackbuttonListener();
        }
    },

    onAppPause: function (evt)
    {
        logger.log('IpPbxMobile - onAppPause');
        try
        {
            if (Ext.isObject(IpPbxMobile.controllers.RemoteContactController) &&
                IpPbxMobile.controllers.RemoteContactController.isUpdateTaskEnabled())
            {
                logger.log('IpPbxMobile - onAppPause - disabling update state task');
                IpPbxMobile.controllers.RemoteContactController.enableUpdateStateTask(false);
            }
        }
        catch (exc)
        {
            // Ignore
            logger.warn('* ERR: ' + exc);
        }
    },

    onAppResume: function (evt)
    {
        logger.log('IpPbxMobile - onAppResume');
        try        
        {
            // Jump out if viewport was not rendered yet
            if (IpPbxMobile.views.viewport == null)
            {
                return;
            }

            var cardId = IpPbxMobile.views.viewport.getCurrentCardId();
            if (cardId === 'contactscard')
            {
                if (Ext.isObject(IpPbxMobile.controllers.RemoteContactController) &&
                    IpPbxMobile.controllers.RemoteContactController.isUpdateTaskEnabled() === false)
                {
                    logger.log('IpPbxMobile - onAppPause - enabling update state task');
                    IpPbxMobile.controllers.RemoteContactController.enableUpdateStateTask(true);
                }
            }

            // Update profile card
            IpPbxMobile.controllers.UserController.getPresenceData(true);
        }
        catch (exc)
        {
            // Ignore
            logger.warn('* ERR: ' + exc);
        }
    },

    onBackButton: function (evt)
    {
        logger.log('IpPbxMobile - onBackButton');

        if (Ext.isObject(IpPbxMobile.views.viewport))
        {
            // Remove this listener
            IpPbxMobile.removeBackbuttonListener();

            // Switch to profile page
            IpPbxMobile.views.viewport.setActiveItem(0);
        }
    }    
});

// Disable Sencha Touch animations globally
Ext.Anim.override({ disableAnimations: true });