//-------------------------------------------------------------------------
// CONFIGURATION - CONTROLLER
//-------------------------------------------------------------------------

IpPbxMobile.controllers.ConfigurationController = new Ext.Controller(
{
    //----------------------------------------------------------------

    // Update the global configData object from the proxy store
    load: function()
    {
        logger.log('ConfigurationController - load');
		
        // Load proxy store
		IpPbxMobile.stores.ConfigItems.load();
        
        // Reset global configData object
        configData = null;
        
        // Check if configuration is valid
        if (IpPbxMobile.stores.ConfigItems.getCount() == 0)
		{
            logger.warn('ConfigurationController - load - No valid config found');
			this.setDefaultConfig();
            return;
		}

        configData = IpPbxMobile.stores.ConfigItems.getAt(0).data;
    },

    //----------------------------------------------------------------
    
    // Save the global configData object from the proxy store
    save: function()
	{
		logger.log('ConfigurationController - save');
			
		IpPbxMobile.stores.ConfigItems.getProxy().clear();
		IpPbxMobile.stores.ConfigItems.load();
		IpPbxMobile.stores.ConfigItems.add(configData);
		IpPbxMobile.stores.ConfigItems.sync();
	},

    //----------------------------------------------------------------
    
    // Sync Store data and update the global configData object
    syncAndReload: function()
	{
		logger.log('ConfigurationController - syncAndReload');
			
		IpPbxMobile.stores.ConfigItems.sync();

        // Reset global configData object
        configData = null;
        configData = IpPbxMobile.stores.ConfigItems.getAt(0).data;
	},
	
	//----------------------------------------------------------------
    // ADDITIONAL INIT METHODS
    //----------------------------------------------------------------

	setDefaultConfig: function()
	{
		logger.log('ConfigurationController - setDefaultConfig');

        if (IpPbxMobile.stores.ConfigItems.getCount() > 0)
        {
            defaultConfig.data.deviceAppId = IpPbxMobile.stores.ConfigItems.getAt(0).data.deviceAppId;
        }

        IpPbxMobile.stores.ConfigItems.getProxy().clear();
		IpPbxMobile.stores.ConfigItems.load();
		IpPbxMobile.stores.ConfigItems.add(defaultConfig.data);
		IpPbxMobile.stores.ConfigItems.sync();

        configData = null;
        configData = IpPbxMobile.stores.ConfigItems.getAt(0).data;
	},
    
	//----------------------------------------------------------------
    // OTHER METHODS
    //----------------------------------------------------------------

    resetConfigForUserSwitch: function()
	{
		logger.log('ConfigurationController - resetConfigForUserSwitch');

        configData.username = defaultConfig.data.username;
        configData.password = defaultConfig.data.password;
        configData.serverId = defaultConfig.data.serverId;

        configData.postDialableDigits = defaultConfig.data.postDialableDigits;

        configData.memNumber = defaultConfig.data.memNumber;
        
        configData.currentUserStateId = defaultConfig.data.currentUserStateId;
        configData.mobileDeviceRegState = defaultConfig.data.mobileDeviceRegState;
        
        configData.firstLogin = true;
        configData.doUserSwitch = true;

		this.save();
	},    
    	
    //----------------------------------------------------------------
    // GENERATE WEBCALL URLs
    //----------------------------------------------------------------

	setWebCallUrls: function()
	{
        logger.log('ConfigurationController - setWebCallUrls');

		webCallUrl['base']           = configData.protocol + "://" + configData.url;
		webCallUrl['ippbxdc']        = webCallUrl['base'] + '/iwa/ippbxdc/main.svc/';        
		webCallUrl['IpPbxUserPics']  = webCallUrl['base'] + '/iwa/userpics/ippbx/';
		webCallUrl['CustomUserPics'] = webCallUrl['base'] + '/iwa/userpics/custom/';	
		
		webCallUrl['ServerList']        = webCallUrl['ippbxdc'] + 'GetServerList';
        webCallUrl['Nonce']             = webCallUrl['ippbxdc'] + 'GetNonce';
		webCallUrl['Login']             = webCallUrl['ippbxdc'] + 'Login';
		webCallUrl['Logout']            = webCallUrl['ippbxdc'] + 'Logout';		
		webCallUrl['UserData']          = webCallUrl['ippbxdc'] + 'GetUserData';
		webCallUrl['Phonebook']         = webCallUrl['ippbxdc'] + 'GetPhonebook';
		webCallUrl['PhonebookStates']   = webCallUrl['ippbxdc'] + 'GetPhonebookStates';		
		webCallUrl['Forwardings']       = webCallUrl['ippbxdc'] + 'GetForwardings';
		webCallUrl['SetForwarding']     = webCallUrl['ippbxdc'] + 'SetForwarding';		
		webCallUrl['CallJournal']       = webCallUrl['ippbxdc'] + 'GetCallJournal';
		webCallUrl['ClearCallJournal']  = webCallUrl['ippbxdc'] + 'ClearCallJournal';
		webCallUrl['PresenceData']      = webCallUrl['ippbxdc'] + 'GetUserPresenceData';
		webCallUrl['SetUserState']      = webCallUrl['ippbxdc'] + 'SetUserState';
        webCallUrl['SetUserFreeText']   = webCallUrl['ippbxdc'] + 'SetUserFreeText';
		webCallUrl['SetDestinationNum'] = webCallUrl['ippbxdc'] + 'SetDestinationNumber';
        webCallUrl['GetPhonebookEntry'] = webCallUrl['ippbxdc'] + 'GetPhonebookEntry';
        webCallUrl['SendMobileAppLog']  = webCallUrl['ippbxdc'] + 'SendMobileAppLog';
        webCallUrl['RegisterDevice']    = webCallUrl['ippbxdc'] + 'RegisterDevice';
        webCallUrl['UnregisterDevice']  = webCallUrl['ippbxdc'] + 'UnregisterDevice';
        webCallUrl['SetUserPicture']    = webCallUrl['ippbxdc'] + 'SetUserPicture';        
	}
		
	//----------------------------------------------------------------
});
