//-------------------------------------------------------------------------
// DIALPAD CONTROLLER
//-------------------------------------------------------------------------

var dialpadAjaxCall = null;

IpPbxMobile.controllers.DialpadController = new Ext.Controller(
{
    //-------------------------------------------------------------------------
    // CALL HANDLING METHODS
    //-------------------------------------------------------------------------

    // Checks the current dial mode and dials the number or shows the dialpad
    handleCallRequest: function(number)
    {
        logger.log('DialpadController - handleCallRequest - DialMode: ' + configData.dialMode);

        // Dial number directly if user wants it that way.
        switch(configData.dialMode)
        {
            //Ask
            case 0:
                // Show dialpad
                IpPbxMobile.views.viewport.setActiveItem(3);
        
                // Dispatch number to dialpad via Controller.
                dialpadNumberField.setText(number);
                dialpadNumberField.resetFontSize();
                break;

            // Office
            case 1:
                // Dial in office mode.
                this.dialOffice(number);
                break;

            // Mobile
            case 2:
                // Dial in Mobile mode.
                this.dialMobile(number);
                break;
        }
    },
    
    //-------------------------------------------------------------------------

    dialMobile: function(destinationNumber)
    {
        var noCheckResult = this.isNumberValid(destinationNumber, false);
        if (noCheckResult !== 0)
        {
            logger.log('Number is invalid (' + destinationNumber + ')');

            // Show error message
            Ext.dispatch(
			{
				controller: IpPbxMobile.controllers.ErrorController,
				action: 'showNumberValidationError',
                errorCode: noCheckResult,
				sourceController: IpPbxMobile.controllers.DialpadController,
				sourceAction: 'dialMobile',
			});

            return;
        }
        
        // Start GSM Call
        this.startCall(destinationNumber, false, false);
    },

    //-------------------------------------------------------------------------

    dialOffice: function(number)
    {
        var noCheckResult = this.isNumberValid(number, true);
        if (noCheckResult !== 0)
        {
            logger.log('Number is invalid (' + number + ')');

            // Show error message
            Ext.dispatch(
			{
				controller: IpPbxMobile.controllers.ErrorController,
				action: 'showNumberValidationError',
                errorCode: noCheckResult,
				sourceController: IpPbxMobile.controllers.DialpadController,
				sourceAction: 'dialOffice',
			});

            return;
        }

        // Check if a valid MEM Number is available
        if (configData.memNumber == "" || configData.memNumber.search(/\+/) == -1 )
        {
            logger.log('MEM Number is invalid (' + configData.memNumber + ')');

            // Show error message
            new Ext.ux.ResultMask(Ext.getBody(),
			{
				msg: langData['dialpad_msg_invalid_mem_number'],
				type: 'error',
				timeout: 3000
			});

            return;
        }

        // Check if Mobility Feature is enabled
        if (!userData.IsMobilityEnabled)
        {
            logger.log('Mobility Feature is disabled for this user');

            // Show error message
            new Ext.ux.ResultMask(Ext.getBody(),
			{
				msg: langData['dialpad_msg_mobility_disabled'],
				type: 'error',
				timeout: 3000
			});

            return;
        }

        // Check if an MobileId is available
        if (!userData.HasMobileId)
        {
            logger.log('User does not have an MobileId');

            // Show error message
            new Ext.ux.ResultMask(Ext.getBody(),
			{
				msg: langData['dialpad_msg_missing_mobileid'],
				type: 'error',
				timeout: 3000
			});

            return;
        }

        // Check if Fast dialup can be used
        var useFastDial = (configData.useFastDial && IsDeviceOnline());

        // Make call:
        this.initOfficeCall(number, useFastDial);
    },
    
    //-------------------------------------------------------------------------
    
    // Makes a call to the given number.
    initOfficeCall: function(destinationNumber, useFastDial)
    {
        logger.log('DialpadController - initOfficeCall (' + destinationNumber + ' / FastDial: ' + useFastDial + ')');

        // Do the default call when fastdialup should not be used:
        if(!useFastDial)
        {
            // Add the number of this call to the offline redial cache
            IpPbxMobile.controllers.JournalController.addRedialCacheEntry(destinationNumber);
            
            // Start Call without Fast Dial
            this.startCall(destinationNumber, true, false);
            return;
        }

        // Abort pending requests
        if (dialpadAjaxCall && dialpadAjaxCall.isLoading())
        {
            logger.warn("DialpadController - initOfficeCall - Request still in Progress");
            return;
        }

        // Init Ajax Call
        dialpadAjaxCall = new Ext.data.Connection(
        {
            url: webCallUrl['SetDestinationNum'],
            timeout: 3000
        });

        dialpadAjaxCall.request(
		{
		    method: 'get',
		    params:
			{
			    n: destinationNumber
			},
		    
            success: function (r)
            {
		        logger.log("DialpadController - SetDestinationNum - Success");

                // Add the number of this call to the offline redial cache
                IpPbxMobile.controllers.JournalController.addRedialCacheEntry(destinationNumber);

                // Start Call with Fast Dial
                IpPbxMobile.controllers.DialpadController.startCall(destinationNumber, true, true);
		    },

		    failure: function (r)
            {
		        logger.warn("DialpadController - SetDestinationNum - Failed");

		        if (r.status !== undefined &&
				   (r.status == 401 || r.status == 403))
                {
		            Ext.dispatch(
				    {
				        controller: IpPbxMobile.controllers.ErrorController,
				        action: 'requestFailed',
				        sourceController: IpPbxMobile.controllers.DialpadController,
				        sourceAction: 'setDestination',
				        isBackgroundProcess: true,
				        statusCode: r.status,
				        statusText: r.statusText
				    });
		        }
		        
                // Fallback to default call scenario (no fast dial up)
                IpPbxMobile.controllers.DialpadController.initOfficeCall(destinationNumber, false);
		    }
		});
    },

    //-------------------------------------------------------------------------
    
    startCall: function(destinationNumber, isOfficeCall, useFastDial)
    {
        logger.log("DialpadController - startCall");

        var handler = "tel:";

        // Use different URI handler for Android 4 devices
        /*if (Ext.ux.is.Android4)
        {
            handler = "wtai://wp/mc;"
        }*/
        
        // Office Call with Fast Dial
        if (isOfficeCall && useFastDial)
        {
            location = handler + configData.memNumber;
            return;
        }

        // Office Call without Fast Dial
        if (isOfficeCall && !useFastDial)
        {
            location = handler + configData.memNumber + this.transformNumber(destinationNumber);            
            return;
        }
        
        // GSM Calls
        if (!isOfficeCall && !useFastDial)
        {
            location = handler + destinationNumber;
            return;
        }
        
        logger.error("DialpadController - startCall - Failed");
    },

    //-------------------------------------------------------------------------
    // GUI INTERACTION METHODS
    //-------------------------------------------------------------------------

    // Number on dialpad was pressed
    touchNumber: function (options)
    {
        dialpadNumberField.setText(dialpadNumberField.getText() + options.data);
        dialpadNumberField.resetFontSize();

        // Reset redial button
        IpPbxMobile.stores.JournalRedialStore.clearData();
    },

    //-------------------------------------------------------------------------

    // Redial Button on dialpad was pressed
    touchRedial: function ()
    {
        if (IpPbxMobile.stores.JournalRedialStore.getCount() === 0)
        {        
            // Add all offline redial cache entries to JournalRedialStore
            IpPbxMobile.stores.JournalRedialStore.add(IpPbxMobile.controllers.JournalController.redialCache);

            // Add all online redial cache entries to JournalRedialStore
            IpPbxMobile.stores.JournalItems.filter(function(record)
            {
                // Type == 2 are Redial Items
                return record.data.Type == 2;
            });

            IpPbxMobile.stores.JournalRedialStore.add(IpPbxMobile.stores.JournalItems.data.items);
            IpPbxMobile.stores.JournalItems.clearFilter(false);
        }

        var newestRedialItem = IpPbxMobile.stores.JournalRedialStore.first();

        if (newestRedialItem != undefined)
        {
            dialpadNumberField.setText(newestRedialItem.data.CalledNumber);
            dialpadNumberField.resetFontSize();
            IpPbxMobile.stores.JournalRedialStore.remove(newestRedialItem);
        }
    },

    //-------------------------------------------------------------------------

    // Event handler from dialpad: Mobile Button on dialpad was pressed
    touchDialMobile: function (options)
    {
        logger.log('Dial Mobile: ' + options.number);

        // Reset redial button
        IpPbxMobile.stores.JournalRedialStore.clearData();

        this.dialMobile(options.number);
    },    

    //-------------------------------------------------------------------------

    // Event handler from dialpad: Office Button on dialpad was pressed
    touchDialOffice: function (options)
    {
        logger.log('Dial Office: ' + options.number);

        // Reset redial button
        IpPbxMobile.stores.JournalRedialStore.clearData();

        this.dialOffice(options.number);
    },
   
    //---------------------------------------------------------------------    
    // NUMBER VALIDATION AND TRANSFORMATION
    //---------------------------------------------------------------------    

    isNumberValid: function(number, isInternalNumberAllowed)
    {
        if (number === undefined)
        {
            return 9; // general error
        }

        // Number given?
        if (number.length < 1)
        {
            // No message box needed
            return 1;
        }

        // Is long enough?
        if (number.length < 2)
        {
            return 2; // Length error
        }

        // Is SIP URI?
        if (number.search(/@/) > -1)
        {
            return 3; // SIP check failed
        }            

        return 0; // no error
    },

    //---------------------------------------------------------------------    

    transformNumber: function (number)
    {
        if (number === undefined)
        {
            return "";
        }

        if (number.search(/\+/) > -1)
        {
            number = number.replace(/\+/gi, '00');
        }

        // Remove obsolete CountryCode
        var fullCountryCode = userData.IntPrefix + userData.CountryCode;

        if (number.slice(0, fullCountryCode.length) == fullCountryCode)
        {
            number = number.substring(fullCountryCode.length);
            number = userData.LongDistPrefix + number;
        }

        // Remove obsolete AreaCode
        var fullAreaCode = userData.LongDistPrefix + userData.AreaCode;

        if (number.slice(0, fullAreaCode.length) == fullAreaCode)
        {
            number = number.substring(fullAreaCode.length);
        }

        // Create comma/semicolon string for post dial digits
        var postDialString = ",";

        if (Ext.ux.is.Android4) 
        {
            postDialString = ";,";
        }

        // Apply max. post dialable digits
        if (configData.postDialableDigits == 0)
        {
            number = postDialString + number;
        }
        else if (configData.postDialableDigits < number.length)
        {
            number = number.substr(0, configData.postDialableDigits) + postDialString + number.substr(configData.postDialableDigits);
        }
        
        return number;
    }

    //---------------------------------------------------------------------
});

 