//-------------------------------------------------------------------------
// ERROR CONTROLLER / ERROR MANAGER
//-------------------------------------------------------------------------

IpPbxMobile.controllers.ErrorController = new Ext.Controller(
{
    // Ajax request has failed (HTTP Code != 200)
    requestFailed: function (options)
    {
        var msg = '';
        var type = '';
        var timeout = 3000;
        var newAuthenticationNeeded = false;
        var serviceUnavailable = false;

        // Check if all required parameters are defined
        if (options.isBackgroundProcess === undefined)
        {
            options.proto.isBackgroundProcess = false;
        }

        if (options.statusCode === undefined ||
            options.statusCode === null)
        {
            options.proto.statusCode = 999;
        }

        switch (options.statusCode)
        {
            // Device is offline (No 3G or Wifi available)  
            case 0:
                msg = langData['msg_server_not_reachable'];
                type = 'error';
                break;

            // BadRequest	  
            case 400:
                msg = langData['msg_bad_request'];
                type = 'error';
                break;

            // Unauthorized	  
            case 401:
                if (options.sourceAction === 'doLogin')
                {
                    msg = langData['msg_login_badprivileges'];
                    timeout = 5000;
                    type = 'error';
                }
                else newAuthenticationNeeded = true;
                break;

            // Forbidden	  
            case 403:
                if (options.sourceAction === 'doLogin')
                {
                    msg = langData['msg_login_invalid'];
                    timeout = 5000;
                    type = 'error';
                }
                else newAuthenticationNeeded = true;
                break;

            // PreconditionFailed (means: Version mismatch)  
            case 412:
                msg = langData['msg_version_mismatch'];
                timeout = 5000;
                type = 'error';
                break;

            // InternalServerError	  
            case 500:
                msg = langData['msg_internal_server_error'];
                type = 'error';
                break;

            // Service Unavailable	  
            case 503:
                serviceUnavailable = true;
                break;

            // Unknown HTTP Error Code received  
            default:
                msg = langData['msg_unknown_error'];
                type = 'error';

                if (options.statusCode)
                    msg += " (" + options.statusCode + ").";
                else msg += " (42).";

                break;
        }

        // Init new user authentication if user session timed out on server side
        if (newAuthenticationNeeded)
        {
            Ext.dispatch(
			{
			    controller: IpPbxMobile.controllers.LoginController,
			    action: 'initRelogin',
			    sourceController: options.sourceController,
			    sourceAction: options.sourceAction,
			    isBackgroundProcess: options.isBackgroundProcess,
			    reLoginCounter: 0,
			    isRelogin: true
			});

            return;
        }

        // DataCollector is waking up - give him enough time
        if (serviceUnavailable)
        {
            var wakeupTask = new Ext.util.DelayedTask(function ()
            {
                Ext.dispatch(
			    {
			        controller: options.sourceController,
			        action: options.sourceAction
			    });
            });

            wakeupTask.delay(2000);
            return;
        }

        // Login in progress - this requires a special handling
        if (!isLoggedOn)
        {
            // Hide login loading mask
            loginMask.hide();

            // Show error message			
            new Ext.ux.ResultMask(Ext.getBody(),
			{
			    msg: msg,
			    type: type,
			    timeout: timeout
			});

            if (startViewLoginFieldset.hidden)
            {
                IpPbxMobile.controllers.StartupController.showLoginView();
            }

            return;
        }

        // No login in process. Proceeding with default error handling

        // Hide every loading screen (mask)
        if (IpPbxMobile.views.viewport !== undefined)
        {
            IpPbxMobile.views.viewport.setLoading(false);
        }

        // Special handling for forwarding configuration errors
        if (options.sourceAction === 'setForwarding')
        {
            IpPbxMobile.controllers.ForwardingController.resetForm();
            msg = langData['forwardings_msg_error'];
            type = 'error';
        }

        // Special handling for user state change errors
        if (options.sourceAction === 'doUserStateChange')
        {
            // Reset content of UserStateChangePage
            IpPbxMobile.controllers.UserController.setUserStateChangePageContent();

            msg = langData['user_card_msg_change_state_failed'];

            if (options.statusCode)
                msg += " (" + options.statusCode + ").";
        }

        // Special handling for status text change errors
        if (options.sourceAction === 'setUserFreeText')
        {
            IpPbxMobile.controllers.UserController.resetUserStatusText();
            msg = langData['user_card_msg_change_statustext_failed'];

            if (options.statusCode)
                msg += " (" + options.statusCode + ").";
        }

        // Show error message
        if (options.isBackgroundProcess === false)
        {
            new Ext.ux.ResultMask(Ext.getBody(),
		    {
		        msg: msg,
		        type: type,
		        timeout: timeout
		    });
        }
    },

    //----------------------------------------------------------------

    reloginFailed: function (options)
    {
        // Check if all required parameters are defined
        if (options.statusCode === undefined ||
            options.statusCode === null)
        {
            options.proto.statusCode = 999;
        }

        // Proceed with next login retry?
        if (options.reloginCounter < 3 && options.statusCode != 403 && options.statusCode !== 0)
        {
            Ext.dispatch(
			{
			    controller: IpPbxMobile.controllers.LoginController,
			    action: 'initRelogin',
			    sourceController: options.sourceController,
			    sourceAction: options.sourceAction,
			    isBackgroundProcess: options.isBackgroundProcess,
			    reloginCounter: options.reloginCounter,
			    isRelogin: true
			});

            return;
        }
        
        // Check if this a background process
        // In this case no error message has to be shown
        if (options.isBackgroundProcess)
        {
            logger.log("Background Process failed (" + options.sourceController + " / " + options.sourceAction + ")");
            return;
        }
                
        // Hide loading mask
        IpPbxMobile.views.viewport.setLoading(false);
        
        // Show error message
        var msg = langData['msg_unknown_error'];

        if (options.statusCode)
            msg += " (" + options.statusCode + ").";
        else msg += " (42).";
        
        new Ext.ux.ResultMask(Ext.getBody(),
		{
			msg: msg,
			type: 'error',
			timeout: 3000
		});
    },

    //----------------------------------------------------------------

    showNumberValidationError: function (options)
    {
        var code = options.errorCode;
        var message = undefined;
        switch (code)
        {
            case 2: // Length error
                message = langData['dialpad_msg_phonenumber_length'];
                break;
            case 3: // SIP check failed
                message = langData['dialpad_msg_sipuri'];
                break;
            case 4: // Internal number cannot be dialed via GSM
                message = langData['dialpad_msg_no_internalnumber'];
                break;
            case 9: // general error
                message = langData['dialpad_msg_invalidphonenumber'];
                break;
        }

        if (message !== undefined)
        {
            // Show message
            new Ext.ux.ResultMask(Ext.getBody(),
			{
			    msg: message,
			    type: 'error',
			    timeout: 3000
			});
        }
    },

    //----------------------------------------------------------------

    loadLocalContactsFailed: function (options)
    {
        // Hide login loading mask
        if (!isLoggedOn)
        {
            loginMask.hide();
        }

        var errorMsg = langData['contacts_msg_localcontacts_failed'];
        errorMsg = errorMsg.replace(/\{0\}/gi, options.message);

        // Show message
        new Ext.ux.ResultMask(Ext.getBody(),
		{
		    msg: errorMsg,
		    type: 'error',
		    timeout: 3000,
		    callback: function ()
		    {
		        if (!isLoggedOn)
		        {
		            IpPbxMobile.controllers.StartupController.loadAppData();
		        }
		    }
		});
    }

    //----------------------------------------------------------------

});


