﻿//-------------------------------------------------------------------------
// CALL JOURNAL CONTROLLER
//-------------------------------------------------------------------------

var journalAjaxCall = null;

IpPbxMobile.controllers.JournalController = new Ext.Controller(
{
    //(all or missed)
    filter: 'all',
	
    // Offline redial cache
    redialCache: [],
    
    //------------------------------------------------------------------------

    //Initialize
    init: function (viewObj)
    {
        logger.log('JournalController - init');
        
        if (viewObj === undefined)
            return;
        
        this.view = viewObj;

        this.view.query('#missedButton')[0].on(
        {
            tap: this.showMissedList,
            scope: this
        });

        this.view.query('#allButton')[0].on(
        {
            tap: this.showAllList,
            scope: this
        });    

        this.view.query('#trashButton')[0].on(
        {
            tap: this.showTrashActionSheet,
            scope: this
        });

        this.view.query('#journallist')[0].on(
        {
            selectionchange: this.handleCallRequest,
            disclose: this.showJournalDetails,
            scope: this
        }); 
    },

    //------------------------------------------------------------------------

    // Show all journalentries in list
    showJournalDetails: function(record, node, index, e)
    {
        logger.log("JournalController - showJournalEntry");
        var journalDetails = record.data;
        switch (record.data.ContactType)
        {
            // Remote contact	                        
            case 1:
                IpPbxMobile.views.journalCard.getComponent(1).setTemplate(remoteJournalContactItemTpl);
                this.updateDetails(journalDetails);
                break;

            // Local contact 
            //TODO Find contact by phonenumber	                         
            /*case 2:
                IpPbxMobile.views.journalCard.getComponent(1).setTemplate(localJournalContactItemTpl);
                break;*/

            // Unknown type or default	                          
            default:
                IpPbxMobile.views.journalCard.getComponent(1).setTemplate(journalContactItemTpl);
                break;
        }
        IpPbxMobile.views.journalCard.getComponent(1).update(journalDetails);
        IpPbxMobile.views.journalCard.setActiveItem(1);
    },

    //------------------------------------------------------------------------
    updateDetails: function (journalEntry)
    {
        logger.log('JournalController - updateDetails');

        if (journalEntry === undefined)
            return;

        // Check type - don't update local phonebook entries.
        if (journalEntry.RemoteContactItem.data.Type === 3) // (Personalphonebookentry == 3)
            return;

        // Abort pending requests
        if (journalAjaxCall && journalAjaxCall.isLoading())
        {
            journalAjaxCall.abort();
            journalAjaxCall = null;
            logger.warn("JournalController - updateDetails - Pending request aborted");
        }

        // Make ajax call
        journalAjaxCall = new Ext.data.Connection();
        journalAjaxCall.request(
		{
		    url: webCallUrl['GetPhonebookEntry'],
		    method: 'get',
		    params:
			{
			    id: journalEntry.RemoteContactItem.data.RefId,
                h: journalEntry.RemoteContactItem.data.PicHash
			},
		    success: function (r)
		    {
		        logger.log("JournalController - updateDetails - Success");

		        // Parse result
		        var recData = JSON.parse(r.responseText);
                
                // Check user picture
                // PicHash == ''              -> No picture found or picture removed
                // PicHash != '' && Pic == '' -> Picture found but was NOT updated
                // PicHash != '' && Pic != '' -> New Picture found
                if (recData.PicHash != '' && recData.Pic == '')
                {                    
                    // Picture is still the same
                    // Copy picture data from current contact
                    recData.Pic = journalEntry.RemoteContactItem.data.Pic;
                }

                // Import result into model
		        var contactData = Ext.ModelMgr.create(recData, 'IpPbxMobile.models.RemoteContactItem');

		        // Check if an update of the view is required
		        if (Ext.encode(journalEntry.RemoteContactItem.data) !== Ext.encode(contactData.data))
		        {
		            var id = contactData.get('RefId');
                    logger.log('JournalController - updateDetails - Updating entry ' + id);

                    // Update UI
                    journalEntry.RemoteContactItem.data = contactData.data;
                    IpPbxMobile.views.journalCard.getComponent(1).updateWithLastComponentSize(journalEntry);

                    // Update store accordingly
                    IpPbxMobile.controllers.RemoteContactController.updateListStoreRecord(id, contactData);
		        }
		    },
		    failure: function (r)
		    {
		        // Trace error
		        logger.log("JournalController - updateDetails - Failed (" + r.status + ')');		       
		    }
		});
    },

    //------------------------------------------------------------------------

    // Handle Call
    handleCallRequest: function (selectionModel, records)
    {
        logger.log('JournalCardController - handleCallRequest');
        if (records[0] != undefined && records[0].data.EntryNumber !== "")
        {
            // Dispatch number to dialpad via Controller
            IpPbxMobile.controllers.DialpadController.handleCallRequest(records[0].data.EntryNumber);
        }

        Ext.getCmp('journallist').deselect(records, false);
    },

    //------------------------------------------------------------------------

    //Show Journallist
    showJournalList: function ()
    {
        logger.log('JournalController - showJournalList');
        IpPbxMobile.views.journalCard.setActiveItem(0);
    },

    //------------------------------------------------------------------------

    // Show all journalentries in list
    showAllList: function ()
    {
        logger.log("JournalController - showAllList");

        if (this.filter == 'all')
            return;

        this.filter = 'all';
        IpPbxMobile.stores.JournalItems.clearFilter(false);
        IpPbxMobile.stores.JournalItems.sort();

        if (Ext.getCmp('journallist').scroller != undefined)
        {
            Ext.getCmp('journallist').scroller.scrollTo(0, null);
        }
    },

    //------------------------------------------------------------------------

    // Show missed journalentries in list
    showMissedList: function ()
    {
        logger.log("JournalController - showMissedList");

        if (this.filter == 'missed')
            return;

        this.filter = 'missed';
        IpPbxMobile.stores.JournalItems.filter({ property: 'IsMissedCall', value: true });
        IpPbxMobile.stores.JournalItems.sort();

        if (Ext.getCmp('journallist').scroller != undefined)
        {
            Ext.getCmp('journallist').scroller.scrollTo(0, null);
        }
    },

    //------------------------------------------------------------------------

    // Clear List
    clearList: function ()
    {
        logger.log("JournalController - clearList");

        // Abort pending requests
        if (journalAjaxCall && journalAjaxCall.isLoading())
        {
            journalAjaxCall.abort();
            journalAjaxCall = null;
            logger.warn("JournalController - clearList - Pending request aborted");
        }

        journalAjaxCall = new Ext.data.Connection();
        journalAjaxCall.request(
		{
		    url: webCallUrl['ClearCallJournal'],
		    method: 'get', 
		    success: function(r)
            {
		        logger.log("JournalController - clearJournal - Success");
				
                // Clear offline redial cache
                IpPbxMobile.controllers.JournalController.clearRedialCache();

		        IpPbxMobile.stores.JournalItems.clearFilter(true);
		        IpPbxMobile.stores.JournalItems.clearData();

		        Ext.getCmp('journallistview').query('#journallist')[0].refresh();
		        Ext.getCmp('journallistview').actionSheet.hide(false);
		    },
		    failure: function(r)
            {
		        logger.warn("JournalController - clearJournal - Failed");
		        Ext.dispatch(
				{
				    controller: IpPbxMobile.controllers.ErrorController,
				    action: 'requestFailed',
				    sourceController: IpPbxMobile.controllers.JournalController,
				    sourceAction: 'clearJournal',
				    statusCode: r.status,
				    statusText: r.statusText
				});
		    }
		});
    },   

    //------------------------------------------------------------------------
    
    // Show TrashActionSheet to empty the journal
    showTrashActionSheet: function()
    {
        logger.log("JournalController - showTrashActionSheet");
        journalActionSheet.show();
    },

    //------------------------------------------------------------------------

    // Get the call journal of the user
    loadJournal: function(isBackgroundProcess)
    {
        logger.log("JournalController - load");

        // Check if call journal should be loaded on app startup
        if (!isLoggedOn &&
            userData.Defaults.LoadJournal != null &&
            userData.Defaults.LoadJournal == false )
        {
            logger.log("JournalController - load -> Skipped");
            IpPbxMobile.controllers.StartupController.loadAppData();
            return;
        }
       
        // Abort pending requests
        if (journalAjaxCall && journalAjaxCall.isLoading())
        {
            journalAjaxCall.abort();
            journalAjaxCall = null;
            logger.warn("JournalController - loadJournal - Pending request aborted");
        }

        // Create new request
        journalAjaxCall = new Ext.data.Connection();        
        journalAjaxCall.request(
		{
		    url: webCallUrl['CallJournal'],
		    method: 'get',
		    params:
			{
			    t: journalTimestamp
			},
		    success: function(r)
            {
		        logger.log("JournalController - loadJournal - Success");

                // Clear offline redial cache
                IpPbxMobile.controllers.JournalController.clearRedialCache();

		        IpPbxMobile.controllers.JournalController.buildList(JSON.parse(r.responseText));
		    },
		    failure: function(r)
            {
		        logger.warn("JournalController - loadJournal - Failed");
		        Ext.dispatch(
				{
				    controller: IpPbxMobile.controllers.ErrorController,
				    action: 'requestFailed',
				    sourceController: IpPbxMobile.controllers.JournalController,
				    sourceAction: 'loadJournal',
                    isBackgroundProcess: isBackgroundProcess,				    
				    statusCode: r.status,
				    statusText: r.statusText
				});
		    }
		});
    },

    //------------------------------------------------------------------------

    // Build call journal
    buildList: function(data)
    {
        logger.log('JournalController - buildList');	          
        
        var appendEntries = true;

        // Check if entries must be appended to the list
        if (journalTimestamp == 0)
        {
            appendEntries = false;            
        }
        
        // Check if entries are available
        if (data.Entries.length > 0)
        {
            IpPbxMobile.stores.JournalItems.clearFilter(false);
            IpPbxMobile.stores.JournalItems.loadData(data.Entries, appendEntries);
            IpPbxMobile.stores.JournalItems.sort();
        }

        // Remember timestamp for next update
        journalTimestamp = data.Timestamp;

        // Hide loading mask if displayed
        if (IpPbxMobile.views.viewport !== undefined && isLoggedOn)
        {
            IpPbxMobile.views.viewport.setLoading(false);
        }

        logger.log('JournalController - buildList - success');

        // Login in process?
        if (!isLoggedOn)
        {
            IpPbxMobile.controllers.StartupController.loadAppData();
            return;
        }

        // If new entries were added ensure that the
        // correct tab is still selected
        if (data.Entries.length > 0 && this.filter != 'all')
        {        
            // Only Missed call are displayed
            IpPbxMobile.stores.JournalItems.filter({ property: 'IsMissedCall', value: true });
            IpPbxMobile.stores.JournalItems.sort();
        }
    },

    //------------------------------------------------------------------------

    addRedialCacheEntry: function(number)
    {
        // Create model data record
        var journalItem = Ext.ModelMgr.create(
        {
            Id: -1,
            Name: '',
            Num: number,
            Date: '',
            DateTS: 0,
            Viewed: 1,
            Callback: 0,
            State: 6,
            Type: 0
        }, 'IpPbxMobile.models.JournalItem');
        
        this.redialCache.unshift(journalItem);
    },
    
    clearRedialCache: function()
    {
        this.redialCache = [];
    }

    //------------------------------------------------------------------------

});
