//-------------------------------------------------------------------------
// LANGUAGE CONTROLLER
//-------------------------------------------------------------------------

IpPbxMobile.controllers.LanguageController = new Ext.Controller(
{
    load: function()
    {
        logger.log('LanguageController - load');

        // Detect current device language
        // This detection is done on every full app start        
        currentLanguage = 'english';

        if (deviceInfo.language.search(/de/) > -1 ||
            deviceInfo.userAgent.search(/de-/) > -1)
        {
            currentLanguage = 'german';
        }

        // Load and parse language XML file
        var langFileUrl = 'app/languages/' + currentLanguage + '.xml';

        var xmlhttp = new XMLHttpRequest();
        xmlhttp.open("GET", langFileUrl, false);
        xmlhttp.setRequestHeader('Content-Type', 'text/xml');
        xmlhttp.send("");
        xmlDoc = xmlhttp.responseXML;

        logger.log("Current Language: " + xmlDoc.documentElement.getAttribute("name"));

        for (var i = 0; i < xmlDoc.getElementsByTagName("entry").length; i++)
        {
            langData[xmlDoc.getElementsByTagName("entry")[i].getAttribute("name")] = xmlDoc.getElementsByTagName("entry")[i].textContent;
        }

    },

    //---------------------------------------------------------------------

    translateStartViews: function ()
    {
        //Common
        Ext.override(Ext.form.Search,
        {
            placeHolder: langData['cn_search']
        });

        Ext.override(Ext.Picker,
        {
            cancelButton: langData['cn_cancel'],
            doneButton: langData['cn_done']
        });

         var msgBox = Ext.MessageBox;

        Ext.apply(msgBox,
        {
            OK     : {text : langData['cn_ok'],     itemId : 'ok',  ui : 'action' },
            CANCEL : {text : langData['cn_cancel'], itemId : 'cancel'},
            YES    : {text : langData['cn_yes'],    itemId : 'yes', ui : 'action' },
            NO     : {text : langData['cn_no'],     itemId : 'no'},
        });

        Ext.apply(msgBox,
        {
            OKCANCEL    : [msgBox.CANCEL, msgBox.OK],
            YESNOCANCEL : [msgBox.CANCEL, msgBox.NO, msgBox.YES],
            YESNO       : [msgBox.YES, msgBox.NO]
        });
       
        // Set Start View Header
        var header = startViewForm.items.get(0).html;
        if (header !== undefined)
        {
            header = header.replace(/appname/, langData['app_name']);
            startViewForm.items.get(0).html = header;
        }

        // Set Start View URL Fieldset title
        startViewUrlField.placeHolder = langData['start_txt_serveraddress'];
        startViewSecurityToggle.setLabel(langData['start_cbx_secureconnection']);        
    },

    //---------------------------------------------------------------------

    translateMainGui: function ()
    {
        // About Card - Footer
        var footer = '';
        footer += '<br><div class="app-rights">v' + appInfoData['appInfo_version'] + '<br>';
        footer +=  langData['app_copyright'] + '<br><br>';
        footer += langData['app_rights'] + '<br><br></div>';
       
        if (langData['app_license_link'] !== '')
        {
            footer += '<div class="app-rights">' + langData['app_license_desc'] + '</div>';
            footer += '<div class="app-rights"><a class="app-rights" href="' + langData['app_license_link'] + '" target="_blank">' + langData['app_license_link'] + '</a></div>';
        }

        IpPbxMobile.views.aboutCard.items.getAt(0).items.getAt(1).html = footer;       

        // My Profile
        IpPbxMobile.views.userCard.title = langData['user_menu_title'];
        IpPbxMobile.views.userCard.query('#profileToolbar')[0].setTitle(langData['user_card_title']);
        IpPbxMobile.views.userCard.query('#statetext')[0].placeHolder = langData['user_card_statustext_placeholder'];
        profileActionSheet.query('#switchUserButtonActionSheet')[0].setText(langData['user_card_btn_switchuser']);
        profileActionSheet.query('#captureButtonActionSheet')[0].setText(langData['user_card_btn_capture_userpic']);        
        profileActionSheet.query('#cancelButtonActionSheet')[0].setText(langData['user_card_btn_cancel']);

        // My Profile - Change State Page
        IpPbxMobile.views.userCard.query('#userStatusToolbar')[0].setTitle(langData['user_card_title_change_state']);
        IpPbxMobile.views.userCard.query('#userStatusBackButton')[0].setText(langData['cn_back']);
        IpPbxMobile.views.userCard.query('#userStateAvailable')[0].setLabel(langData['state_available']);
        IpPbxMobile.views.userCard.query('#userStateAway')[0].setLabel(langData['state_away']);
        IpPbxMobile.views.userCard.query('#userStateDND')[0].setLabel(langData['state_dnd']);
        IpPbxMobile.views.userCard.query('#userStateNotVisible')[0].setLabel(langData['state_notvisible']);

        // Contacts
        IpPbxMobile.views.contactsCard.title = langData['contacts_menu_title'];
        IpPbxMobile.views.contactsCard.query('#localContactBackButton')[0].text = langData['contacts_back'];
        IpPbxMobile.views.contactsCard.query('#remoteContactBackButton')[0].text = langData['contacts_back'];
        IpPbxMobile.views.contactsCard.query('#titleRemoteContact')[0].title = langData['contacts_detail_title'];
        IpPbxMobile.views.contactsCard.query('#titleLocalContact').title = langData['contacts_detail_title'];
        
        // Call Journal
        IpPbxMobile.views.journalCard.title = langData['journal_menu_title'];
        IpPbxMobile.views.journalCard.query('#missedButton')[0].setText(langData['journal_btn_missed']);
        IpPbxMobile.views.journalCard.query('#allButton')[0].setText(langData['journal_btn_all']);        
        IpPbxMobile.views.journalCard.query('#backButton')[0].setText(langData['journal_btn_back']);
        IpPbxMobile.views.journalCard.query('#titleJournalDetails')[0].title = langData['journal_detail_title'];
        journalActionSheet.query('#clearJournalActionSheetBtn')[0].setText(langData['journal_btn_removeentries']);
        journalActionSheet.query('#cancelJournalActionSheetBtn')[0].setText(langData['journal_btn_cancel']);
        
        // Dialpad
        IpPbxMobile.views.dialpadCard.title = langData['dialpad_menu_title'];
        IpPbxMobile.views.dialpadCard.rendered = false;

        // More
        IpPbxMobile.views.moreCard.title = langData['more_menu_title'];
        moreToolbar.setTitle(langData['more_card_title']);
        moreCardBackButton[0].text = langData['cn_back'];

        // More List Item - Settings
        IpPbxMobile.stores.MoreListItems.getAt(0).data.text = langData['settings_list_title'];
        IpPbxMobile.stores.MoreListItems.getAt(0).data.title = langData['settings_card_title'];

        IpPbxMobile.views.settingsCard.query('#useAutoLogin')[0].setLabel(langData['settings_autologin']);
        IpPbxMobile.views.settingsCard.query('#useFastDial')[0].setLabel(langData['settings_fast_dial']);
        IpPbxMobile.views.settingsCard.query('#postDialableDigits')[0].setLabel(langData['settings_max_postdigits']);
        IpPbxMobile.views.settingsCard.query('#memNumber')[0].setLabel(langData['settings_number']);
        IpPbxMobile.views.settingsCard.query('#useLogging')[0].setLabel(langData['settings_logging']);        
        IpPbxMobile.views.settingsCard.query('#fieldsetDiagnostics')[0].setTitle(langData['settings_diagnostics']);

        var dialModeControl = IpPbxMobile.views.settingsCard.query('#dialMode')[0];
        dialModeControl.setLabel(langData['settings_dialmode']);
        dialModeControl.options[0].data.text = langData['settings_dialmode_ask'];
        dialModeControl.options[1].data.text = langData['settings_dialmode_office'];
        dialModeControl.options[2].data.text = langData['settings_dialmode_mobile'];
        
        settingsActionSheet.query('#resetSettingsButtonActionSheet')[0].setText(langData['settings_btn_resetsettings']);        
        settingsActionSheet.query('#sendLogButtonActionSheet')[0].setText(langData['settings_btn_send_log']);
        settingsActionSheet.query('#cancelButtonActionSheet')[0].setText(langData['settings_btn_cancel']);        

        // More List Item - Forwarding List
        IpPbxMobile.stores.MoreListItems.getAt(1).data.text = langData['forwardings_list_title'];
        IpPbxMobile.stores.MoreListItems.getAt(1).data.title = langData['forwardings_card_title'];
        fwdDetailBackButton[0].text = langData['cn_back'];

        // More List Item - Forwarding Details - Busy
        var fwdBusyCardFieldset = IpPbxMobile.views.forwardingDetailCards[0].items.getAt(0);
        fwdBusyCardFieldset.query('#fwdEnabled')[0].setLabel(langData['forwardings_details_activate']);
        fwdBusyCardFieldset.query('#fwdToVoicemail')[0].setLabel(langData['forwardings_details_isvoicemail']);
        fwdBusyCardFieldset.query('#fwdNumber')[0].setLabel(langData['forwardings_details_isNumber']);

        // More List Item - Forwarding Details - NoReply
        var fwdNoReplyCardFieldset = IpPbxMobile.views.forwardingDetailCards[1].items.getAt(0);
        fwdNoReplyCardFieldset.query('#fwdEnabled')[0].setLabel(langData['forwardings_details_activate']);
        fwdNoReplyCardFieldset.query('#fwdToVoicemail')[0].setLabel(langData['forwardings_details_isvoicemail']);
        fwdNoReplyCardFieldset.query('#fwdNumber')[0].setLabel(langData['forwardings_details_isNumber']);
        fwdNoReplyCardFieldset.query('#fwdWaitTime')[0].setLabel(langData['forwardings_details_waitCall']);

        // More List Item - Forwarding Details - Unconditional
        var fwdUnconditionalCardFieldset = IpPbxMobile.views.forwardingDetailCards[2].items.getAt(0);
        fwdUnconditionalCardFieldset.query('#fwdEnabled')[0].setLabel(langData['forwardings_details_activate']);
        fwdUnconditionalCardFieldset.query('#fwdToVoicemail')[0].setLabel(langData['forwardings_details_isvoicemail']);
        fwdUnconditionalCardFieldset.query('#fwdNumber')[0].setLabel(langData['forwardings_details_isNumber']);

        // More List Item - Help (Hidden by default)        
        if (IpPbxMobile.stores.MoreListItems.getById('help'))
        {
            IpPbxMobile.stores.MoreListItems.getById('help').data.text = langData['help_list_title'];
            IpPbxMobile.stores.MoreListItems.getById('help').data.title = langData['help_card_title'];
        }

        // More List Item - Info
        if (IpPbxMobile.stores.MoreListItems.getById('about'))
        {
            IpPbxMobile.stores.MoreListItems.getById('about').data.text = langData['about_list_title'];
            IpPbxMobile.stores.MoreListItems.getById('about').data.title = langData['about_card_title'];
        }
                
        // More List Item - Credits
        try
        {
            if (creditsHtml.length > 7)
            {
                creditsHtml[6] = creditsHtml[6].replace(/#apptitle#/, langData['app_name']);
                creditsHtml[7] = creditsHtml[7].replace(/#appversion#/, appInfoData['appInfo_version']);
            }
        }
        catch(exc)
        {
            logger.warn('Credits translation failed');
            logger.warn('*ERR: ' + exc);
        }
    },

    //---------------------------------------------------------------------

    getLocalisedStatusText: function (statusText)
    {
        statusText = statusText.replace("%ab%", langData['user_card_AppointmentBusyText']);
        statusText = statusText.replace("%ao%", langData['user_card_AppointmentOutOfOfficeText']);
        statusText = statusText.replace("%ap%", langData['user_card_AppointmentPrivateText']);

        return statusText;
    },
    
    //---------------------------------------------------------------------

    getLocalisedStatusName: function (stateId, isOwnUserState)
    {
        switch (stateId)
        {
            case 0: 
                if (isOwnUserState)
                    return langData['state_notvisible'];
                else return langData['state_loggedoff'];

            case 1: return langData['state_available'];
            case 2: return langData['state_busy'];
            case 3: return langData['state_nostate'];
            case 4: return langData['state_away'];
            case 5: return langData['state_dnd'];
        }

        return langData['state_nostate'];
    },    

    //---------------------------------------------------------------------

    getLocalisedCallStateName: function (callType, callStateId)
    {
        // Outgoing Call  
        if (callType == 2)
        {
            return langData['journal_state_Outgoing'];
        }
                
        // This must be an Incoming Call

        // Call State Id Translation Table
        // 0 = Accepted
        // 1 = NotAccepted
        // 2 = AskedForCallback
        // 3 = CalledBack
        // 4 = Diverted
        // 5 = DivertedVoicemail
        // 6 = Unknown
        // 7 = AcceptedByTransfer

        switch (callStateId)
        {
            case 0: return langData['journal_state_Accepted'];
            case 1: return langData['journal_state_NotAccepted'];
            case 2: return langData['journal_state_AskedForCallback'];
            case 3: return langData['journal_state_CalledBack'];
            case 4: return langData['journal_state_Diverted'];
            case 5: return langData['journal_state_DivertedVoicemail'];
            case 7: return langData['journal_state_AcceptedByTransfer'];
        }

        // Case 6 is also call state "unknown"
        return langData['journal_state_Unknown']; // Unknown Call State
    },

    //---------------------------------------------------------------------

    getLocalisedFwdTypeName: function(typeId)
    {
        switch (typeId)
        {
            case 0:
                return langData['forwardings_busy_title'];
            case 1:
                return langData['forwardings_noreply_title'];
            case 2:
                return langData['forwardings_unconditional_title'];
        }
    },    

    //---------------------------------------------------------------------

    setFieldLabelFromCard: function (detailCard, fieldName, fieldValue)
    {
        var fields = detailCard.items.items[0].fields;
        for (i = 0; i < fields.length; i++)
        {
            var field = fields.items[i];
            if (field.name == fieldName)
            {
                field.setLabel(fieldValue);
                detailCard.doLayout();
                break;
            }
        }
    }
});

//-------------------------------------------------------------------------

