﻿/**
* @class IpPbxMobile.controllers.LocalContactController
* The Controller for the local contactlist 
*/

IpPbxMobile.controllers.LocalContactController = new Ext.Controller(
{
    currentListMask: null,

    listControl: null,

    //Initialize
    init: function (options)
    {
        logger.log('LocalContactController - init');

        if (options != undefined)
        {
            logger.log('LocalContactController -init');
            this.view = options.data;

            this.view.query('#remoteButton')[0].on(
            {
                tap: this.showRemoteList,
                scope: this
            });

            this.view.query('#localButton')[0].on(
            {
                tap: this.showLocalList,
                scope: this
            });

            this.view.query('#refreshButton')[0].on(
            {
                tap: this.refreshContacts,
                scope: this
            });

            this.view.query('#localList')[0].on(
            {
                selectionchange: this.showContact
            });


            this.view.query('#localSearchField')[0].on(
            {
                keyup: this.searchLocalList,
                scope: this
            });
        }
    },

    //---------------------------------------------------------------------

    ensureValidListControlRef: function ()
    {
        // Update list control reference
        if (this.listControl === null)
            this.listControl = this.view.query('#localList')[0];
    },

    // ---------------------------------------------------------------------
    // Scrolls the list to top (handles a buggy behavior in Sencha Touch
    // List)
    refreshListControl: function ()
    {
        logger.log('LocalContactController - refreshListControl');
        this.ensureValidListControlRef();
        this.listControl.scroller.scrollTo({ x: 0, y: 0 });
    },

    // ---------------------------------------------------------------------
    //Clear local list after delete button click
    clearLocalList: function ()
    {
        logger.log('LocalContactController - clearLocalList');
        this.view.query('#localSearchField')[0].setValue('')
        Ext.getStore('LocalContactItemStore').clearFilter(true);
        Ext.getStore('LocalContactItemStore').filter('displayName', '');
        this.refreshListControl();
    },
    // ---------------------------------------------------------------------

    //Search Local List
    searchLocalList: function (text)
    {
        logger.log('LocalContactController - searchLocalList');

        var store = Ext.getStore('LocalContactItemStore');
        if (store)
        {
            store.clearFilter(true);

            var re = null;
            try
            {
                re = new RegExp(text.getValue(), "i");
            }
            catch (exception)
            {
                re = new RegExp("\\" + text.getValue(), "i");
            }

            if (re != null)
            {
                store.filter('displayName', re);
                this.refreshListControl();
            }
        }
    },

    // ---------------------------------------------------------------------

    //Show Remote List
    showRemoteList: function ()
    {
        logger.log('LocalContactController - showRemoteList');

        // Update contact states
        Ext.dispatch(
		{
		    controller: IpPbxMobile.controllers.RemoteContactController,
		    action: 'updateList'
		});

        IpPbxMobile.views.contactscard.setActiveItem(0);
    },

    // ---------------------------------------------------------------------

    //Show Local List
    showLocalList: function ()
    {
        logger.log('LocalContactController - showLocalList');
        IpPbxMobile.views.contactscard.setActiveItem(2);
    },

    // ---------------------------------------------------------------------

    // Show ContactDetails
    showContact: function (selectionModel, records)
    {
        logger.log('LocalContactController - showContact');
        if (records != undefined)
        {
            var contact = records[0].data;

            // Show contact details.
            Ext.getCmp('localcontactdetailsview').update(contact);
            IpPbxMobile.views.contactscard.setActiveItem(3);
        }
        selectionModel.deselect(records, true);
    },

    //---------------------------------------------------------------------

    showLoadingMask: function (m)
    {
        // Delete any previous loading mask.
        this.hideLoadingMask();

        // Show Loading Mask
        this.currentListMask = new Ext.LoadMask(Ext.getBody(), { msg: m });
        this.currentListMask.show();
    },

    //---------------------------------------------------------------------

    hideLoadingMask: function ()
    {
        try
        {
            // Hide Loading Mask
            if (this.currentListMask)
                this.currentListMask.destroy();
        }
        finally
        {
            this.currentListMask = null;
        }
    },

    // ---------------------------------------------------------------------

    refreshContacts: function ()
    {
        this.showLoadingMask(langData['msg_load_phone_contacts']);
        this.loadContacts();
    },

    // ---------------------------------------------------------------------

    // Load local contacts from physical phone device
    loadContacts: function ()
    {
        logger.log('LocalContactController - loadContacts');

        // Check if phone contacts should be loaded on app startup
        if (!isLoggedOn &&
            userData.Defaults.LoadPhone != null &&
            userData.Defaults.LoadPhone == false )
        {
            logger.log("LocalContactController - loadContacts -> Skipped");
            IpPbxMobile.controllers.StartupController.loadAppData();
            return;
        }

        try
        {
            // App running on device?
            if (navigator.contacts === undefined)
            {
                logger.warn("LocalContactController - loadContacts - No physical device found");

                if (!isLoggedOn)
                {
                    IpPbxMobile.controllers.StartupController.loadAppData();
                }
                return;
            }

            logger.log('LocalContactController - loadContacts - contacts.find (navigator.contacts = ' + navigator.contacts + ')');

            // Phonegap is crashing on some iPhones when the photos field is given to
            // navigator.contacts.find() (this happens with e.g. Xing and/or LinkedIn
            // PlugIn). For now we just add the photos field in case of Android.
            var contactFields = ['id', 'name', 'emails', 'phoneNumbers', 'organizations'];

            if (Ext.is.Android)
            {
                contactFields.push('photos');
            }

            navigator.contacts.find(contactFields,
                                            function (contacts)
                                            {
                                                logger.log('LocalContactController - loadContacts - contacts.find LocalContactController buildList');

                                                Ext.dispatch(
                                                {
                                                    controller: IpPbxMobile.controllers.LocalContactController,
                                                    action: 'buildList',
                                                    contacts: contacts
                                                });
                                            },
						                    function (error)
						                    {
						                        logger.warn('LocalContactController - loadContacts - contacts.find LocalContactController loadLocalContactsFailed ' + error);

						                        IpPbxMobile.controllers.LocalContactController.hideLoadingMask();

						                        Ext.dispatch(
                                                {
                                                    controller: IpPbxMobile.controllers.ErrorController,
                                                    action: 'loadLocalContactsFailed',
                                                    message: error
                                                });
						                    },
                                            {
                                                filter: "",
                                                multiple: true
                                            });
        }
        catch (exc)
        {
            logger.error(exc);
            //alert('* EXC: ' + exc);
        }
    },

    // ---------------------------------------------------------------------

    // Returns the pref entry of a collection or the first item of the collection if
    // no pref entry could be found (pref is not supported on some devices).
    getPrefEntry: function (coll)
    {
        var res = null;

        if (Ext.isArray(coll) && coll.length > 0)
        {
            // Check for pref property
            for (var i = 0; i < coll.length; i++)
            {
                var item = coll[i];

                if (item && item.pref === true)
                {
                    res = item;
                    break;
                }
            }

            // No pref found?
            if (res === null)
            {
                // Return first item
                res = coll[0];
            }
        }

        return res;
    },

    // ---------------------------------------------------------------------

    // Returns an empty array if parameter has an invalid value.
    getSafeArray: function (obj)
    {
        if (obj != null && Ext.isArray(obj))
            return obj;
        return [];
    },

    // ---------------------------------------------------------------------

    buildList: function (options)
    {
        var contacts = [];

        logger.log("LocalContactController - buildList - Success");

        try
        {

            if (options.contacts != null && Ext.isArray(options.contacts))
            {
                //alert(options.contacts.length);

                //for (var i = 0; i < 3; i++)
                for (var i = 0; i < options.contacts.length; i++)
                {
                    try
                    {
                        var deviceContact = options.contacts[i];

                        /*
                        alert(deviceContact.name.givenName + " " + deviceContact.name.familyName);

                        alert(deviceContact.phoneNumbers.length);
                        if (deviceContact.phoneNumbers.length > 0)
                        {
                        alert(deviceContact.phoneNumbers[0].pref); // A string that tells you what type of field this is (example: 'home'). (DOMString)
                        alert(deviceContact.phoneNumbers[0].type); // The value of the field (such as a phone number or email address). (DOMString)
                        alert(deviceContact.phoneNumbers[0].value); // Set to true if this ContactField contains the user's preferred value. (boolean)
                        }

                        alert(deviceContact.emails.length);
                        if (deviceContact.emails.length > 0)
                        {
                        alert(deviceContact.emails[0].pref); // A string that tells you what type of field this is (example: 'home'). (DOMString)
                        alert(deviceContact.emails[0].type); // The value of the field (such as a phone number or email address). (DOMString)
                        alert(deviceContact.emails[0].value); // Set to true if this ContactField contains the user's preferred value. (boolean)
                        }

                        alert(deviceContact.photos.length);
                        if (deviceContact.photos.length > 0)
                        {
                        alert(deviceContact.photos[0].pref); // A string that tells you what type of field this is (example: 'home'). (DOMString)
                        alert(deviceContact.photos[0].type); // The value of the field (such as a phone number or email address). (DOMString)
                        alert(deviceContact.photos[0].value); // Set to true if this ContactField contains the user's preferred value. (boolean)
                        }

                        alert(deviceContact.organizations.length);
                        if (deviceContact.organizations.length > 0)
                        {
                        alert(deviceContact.organizations[0].pref); // Set to true if this ContactOrganization contains the user's preferred value. (boolean)
                        alert(deviceContact.organizations[0].type); // A string that tells you what type of field this is (example: 'home'). _(DOMString)
                        alert(deviceContact.organizations[0].name); // The name of the organization. (DOMString)
                        alert(deviceContact.organizations[0].department); // The department the contract works for. (DOMString)
                        alert(deviceContact.organizations[0].title); // The contacts title at the organization. (DOMString)
                        }
                        */

                        var mainCompany = this.getPrefEntry(deviceContact.organizations);
                        var mainJobTitle = (mainCompany == null ? '' : mainCompany.title);
                        var mainPhoto = this.getPrefEntry(deviceContact.photos);

                        // Filter out fax number and invalid items
                        var phoneNumberList = [];
                        if (Ext.isArray(deviceContact.phoneNumbers) &&
                             deviceContact.phoneNumbers.length > 0)
                        {
                            var noLen = deviceContact.phoneNumbers.length;
                            for (var n = 0; n < noLen; n++)
                            {
                                var phoneNoItem = deviceContact.phoneNumbers[n];
                                if (phoneNoItem != null &&
                                    phoneNoItem.type != null &&
                                    (phoneNoItem.type.toLowerCase().indexOf('fax') === -1) &&
                                    phoneNoItem.value != null)
                                {
                                    phoneNumberList.push(phoneNoItem);
                                }
                            }
                        }

                        var givName = (deviceContact.name != null) ? deviceContact.name.givenName : '';
                        var famName = (deviceContact.name != null) ? deviceContact.name.familyName : '';

                        // Create model data record
                        var contact = Ext.ModelMgr.create(
                        {
                            id: deviceContact.id,
                            givenName: givName,
                            familyName: famName,
                            emails: this.getSafeArray(deviceContact.emails),
                            phoneNumbers: phoneNumberList,
                            organizations: this.getSafeArray(deviceContact.organizations),
                            photos: this.getSafeArray(deviceContact.photos),
                            jobTitle: mainJobTitle,
                            company: ((mainCompany == null || mainCompany.name == null) ? '' : mainCompany.name),
                            displayPhoto: ((mainPhoto == null || mainPhoto.value == null) ? '' : mainPhoto.value)

                        }, 'IpPbxMobile.models.LocalContactItem');

                        contacts.push(contact);
                    }
                    catch (errObj)
                    {
                        logger.error('*** Exception: ' + errObj);
                        //alert('*** Exception: ' + errObj);
                    }
                }

                IpPbxMobile.stores.LocalContactItems.clearFilter(true);
                IpPbxMobile.stores.LocalContactItems.loadData(contacts, false);

                IpPbxMobile.stores.LocalContactItems.sort('sortField', 'ASC');
            }
            else
            {
                logger.log("LocalContactController - buildList - No contacts received");
            }
        }
        finally
        {
            IpPbxMobile.controllers.LocalContactController.hideLoadingMask()
        }

        logger.log('LocalContactController - buildList - success');

        if (!isLoggedOn)
        {
            IpPbxMobile.controllers.StartupController.loadAppData();
        }
    }

    // ---------------------------------------------------------------------
});
