//-------------------------------------------------------------------------
// LOGGING - CONTROLLER
//-------------------------------------------------------------------------

var sendMobileLogAjaxCall = null;

IpPbxMobile.controllers.LoggingController = Ext.extend(Ext.Controller,
{	
    isEnabled: false,    
    consoleOutput: true,    
    logData: null,

    //----------------------------------------------------------------

    constructor: function(config)
    {
        // Call base constructor
        IpPbxMobile.controllers.LoggingController.superclass.constructor.call(this, config);

        // Create the log data array
        this.logData = new Array();

        // Logger must be disabled by default
        this.isEnabled = false;
        
        // Check if logger is enabled 
		IpPbxMobile.stores.ConfigItems.load();		
        if (IpPbxMobile.stores.ConfigItems.getCount() > 0)
		{
            this.isEnabled = IpPbxMobile.stores.ConfigItems.getAt(0).data.useLogging;
        }
		IpPbxMobile.stores.ConfigItems.clearData();

        // Write logger start message if enabled
        if (this.isEnabled)
        {
            // Hey I am alive. This must be the headline of every newspaper :-)
            this.writeLog("log", "LoggingController - init - Logging enabled");
        }
    },  

    //----------------------------------------------------------------

    update: function()
    {
        // Check if a vaild configuration is available
        if (IpPbxMobile.stores.ConfigItems.getCount() > 0)
        {
            this.isEnabled = IpPbxMobile.stores.ConfigItems.getAt(0).data.useLogging;
        }
        else
        {
            // No valid config available
            // Logger will be disabled
            this.isEnabled = false;
        }
        
        // If Logger was disabled the log card has to be cleared
        if (!this.isEnabled)
        {
            this.clearLog();
            return;
        }

        this.writeLog("log", "LoggingController - update - Logging enabled");
    },

    //----------------------------------------------------------------

    clearLog: function()
    {
        this.logData.splice(0);
    },
	
	//----------------------------------------------------------------
    // Logging Methods
    //----------------------------------------------------------------
	
	log: function(message)
	{
        this.writeLog("log", message);
	},
	
	warn: function(message)
	{
		this.writeLog("warn", message);
	},
	
	error: function(message)
	{
		this.writeLog("error", message);
	},

    //----------------------------------------------------------------

    writeLog: function(level, message)
	{
        // Check if logging is enabled
        // The IsEnabled flag is ignored for Errors and Warnings
        if (!this.isEnabled && 
             level !== "warn" && 
             level !== "error")
            return;

        if (this.consoleOutput)
        {
            switch(level)
            {
                case "warn":
                    console.warn(message);
                    break;
                case "error":
                    console.error(message);
                    break;
                default:
                    console.log(message);
                    break;
            }
        }

        // Check if logging is enabled
        if (!this.isEnabled)
            return;

        this.logData.push({
            ts: getTimeString(),
            level: level,
            msg: message
        });
	},
	
    //----------------------------------------------------------------   

    sendMobileLog: function()
    {
        if(!Ext.isArray(this.logData) || this.logData.length <= 0)
        {
            // There is nothing to send
			// ATTENTION: Don't use logger here - we would create new entries
			// in logData which would be slightly stupid here... ;-)
            return;
        }

        var logMask = new Ext.LoadMask(Ext.getBody(), { msg: langData['settings_msg_sending_log'] });
        logMask.show();
        
        // Init Ajax Call
        sendMobileLogAjaxCall = null;        
        sendMobileLogAjaxCall = new Ext.data.Connection(
        {
            url: webCallUrl['SendMobileAppLog'],
            timeout: 20000
        });

        sendMobileLogAjaxCall.request(
		{
		    method: 'post',
            headers:
            {
                'Content-Type': 'application/json;charset=utf-8'
            },
		    params: Ext.encode(this.logData),
            scope: this,
		    success: function (r)
		    {
		        // Hide Mask
                logMask.hide();
                
                // Clear log data on success
                this.clearLog();                
		    },
		    failure: function (r)
		    {
                // Hide Mask
                logMask.hide();
                
                Ext.dispatch(
				{
				    controller: IpPbxMobile.controllers.ErrorController,
				    action: 'requestFailed',
				    sourceController: IpPbxMobile.controllers.LoggingController,
				    sourceAction: 'sendMobileLog',
				    statusCode: r.status,
				    statusText: r.statusText
				});
		    }
		});
    }

    //----------------------------------------------------------------   
});
