//-------------------------------------------------------------------------
// LOGIN SETTINGS CONTROLLER
//-------------------------------------------------------------------------

var loginAjaxCall=null;

IpPbxMobile.controllers.LoginController = new Ext.Controller(
{
	checkNewUrl: function(options)
    {
        logger.log("LoginController - checkNewUrl");
        
        // Show login loading mask
    	loginMask.msg = langData['msg_connecting'];
	    loginMask.show();
    	
        // Check if field is empty
        if (options.url == "")
        {
            // Show error message			
			new Ext.ux.ResultMask(Ext.getBody(),
			{
				msg: langData['msg_login_missingurl'],
				type: 'error',
				timeout: 3000
			});
            return;
        }

        // Check if a all needed parameters are available
		if (options.protocol === undefined)
			options.proto.protocol = ""; 
            
		urlCheckCount++;
		
        // Trim URL        
        options.url = Ext.util.Format.trim(options.url);

		// Given URL contains a protocol (e.g. http)
		if (options.url.indexOf("://") > -1)    	
		{
            urlCheckCount = 2;
			options.protocol = options.url.split("://")[0];			
			options.url = options.url.split("://")[1];		
			logger.log("LoginController - checkNewUrl - URL contains protocol (" + options.protocol + ")");
    	}
        else if (options.useSecureConnection)
		{			
			urlCheckCount = 2;
            logger.log("LoginController - checkNewUrl - Secure Connection forced - Trying HTTPS only");
			options.protocol = "https";
		}
		else if (options.protocol === "")
		{			
			logger.log("LoginController - checkNewUrl - No Protocol found. Trying HTTPS");
			options.protocol = "https";
		}
		else
		{
			logger.log("LoginController - checkNewUrl - HTTPS does not work. Trying HTTP");
			options.protocol = "http";
		}	
		
		// Remove slash at the end of the URL
		if (options.url[options.url.length-1] == "/")
		{
			options.url.substring(0,options.url.length-2);
		}
		
        // Save URL and protocol
        configData.protocol = options.protocol;
        configData.url = options.url;
        configData.useSecureConnection = options.useSecureConnection;        
        IpPbxMobile.controllers.ConfigurationController.save();

		// Generate WebCall URLs
		IpPbxMobile.controllers.ConfigurationController.setWebCallUrls();

    	this.connect();
    },  
  
    //----------------------------------------------------------------
    
    // Check WebAccess URL and receive ServerList
	connect: function()
    {
		logger.log("LoginController - connect - (" + webCallUrl['base'] + ")");
		        
        // Abort pending requests
        if (loginAjaxCall && loginAjaxCall.isLoading())
        {
            loginAjaxCall.abort();
            loginAjaxCall = null;
            logger.warn("LoginController - connect - Pending request aborted");
        }

        // Init Ajax Call
        loginAjaxCall = new Ext.data.Connection(
        {
            url: webCallUrl['ServerList'],
            timeout: 4000
        });

		loginAjaxCall.request(
		{
			method: 'get',
			success: function(r, o)
			{
			    logger.log("LoginController - connect - Success");

	        	urlCheckCount = 0;
	        	
	        	// Convert and save Server List Data
	        	IpPbxMobile.controllers.LoginController.saveServerList(JSON.parse(r.responseText));
	        	
                // Hide login loading mask
           	    loginMask.hide();
        		
                // Check configuration data
                if (configData.useAutoLogin &&
                   !configData.doUserSwitch &&
                   configData.username != '' &&
                   configData.password != '')
                {
                    Ext.dispatch(
    			    {
    			        controller: IpPbxMobile.controllers.LoginController,
    			        action: 'initLogin',
                        useAutoLogin: true
    			    });

                    return;
                }

                // Reset user switch flag
                if (configData.doUserSwitch)
                {
                    configData.doUserSwitch = false;        
		            IpPbxMobile.controllers.ConfigurationController.save();
                }
                
                // Show login form
                IpPbxMobile.controllers.StartupController.showLoginView();
			},
			failure: function(r, o)
			{
			    logger.warn("LoginController - connect - Failed");
				
                // Hide login loading mask
        	    loginMask.hide();
				
                if (urlCheckCount > 1)
				{				
					urlCheckCount = 0;
					
					new Ext.ux.ResultMask(Ext.getBody(),
					{
						msg: langData['msg_connection_failed'],
						type: 'error',
						timeout: 3000
					});	
					
		        	IpPbxMobile.controllers.StartupController.showUrlView();
				}
                else
                {
		            Ext.dispatch(
			        {
				        controller: IpPbxMobile.controllers.LoginController,
				        action: 'checkNewUrl',
				        url: configData.url,
				        protocol: configData.protocol,
                        useSecureConnection: configData.useSecureConnection
			        });
                }
			}
		});    	
		
    },
    
    //----------------------------------------------------------------

    // User login at the DataCollector 
    // This method is only called by the login form
    initLogin: function(options)
    {
      	logger.log("LoginController - login");

        // Get user login credentials from config store
        if (options.username === undefined) options.proto.username = configData.username;
        if (options.password === undefined) options.proto.password = configData.password;
        if (options.serverId === undefined) options.proto.serverId = configData.serverId;

        // Check if fields are empty
        if (options.username == "" || options.password == "")
        {
            // Show error message			
			new Ext.ux.ResultMask(Ext.getBody(),
			{
				msg: langData['msg_login_missingdata'],
				type: 'error',
				timeout: 3000
			});
            
            return;
        }
 
        // Show login loading mask
    	loginMask.msg = langData['msg_logging_on'];
	    loginMask.show();

		// Reset login progress state indicator
        nextLoginStep = null;
	   		
        // Auto login variable defined?
        if (options.isRelogin === undefined)
        {
            options.proto.isRelogin = false;
        }
        else options.isRelogin = false;

        // Start login process
        this.doLogin(options);       
    },

    //----------------------------------------------------------------

    // Background user login at the DataCollector 
    // This method is only called if a web call fails
    // with one of the following HTTP error codes:
    // 401 Unauthorized
    // 403 Forbidden
    initRelogin: function(options)
    {
        // Auto login variable defined?
        if (options.isRelogin === undefined)
        {
            options.proto.isRelogin = true;
        }
        else options.isRelogin = true;
        
        // Get user login credentials from config store
        if (options.username === undefined) options.proto.username = "";
        if (options.password === undefined) options.proto.password = "";
        if (options.serverId === undefined) options.proto.serverId = 0;

        options.username = configData.username;
        options.password = configData.password;
        options.serverId = configData.serverId;

        // Start login process
        this.doLogin(options);
    },
    
    //----------------------------------------------------------------
    
    // Try to logon at the IpPbx DataCollector
    doLogin: function(options)
    {    	
        logger.log("LoginController - doLogin");
                      
        // Check if all required parameters are defined
        if (options.isBackgroundProcess === undefined)
        {        
            options.proto.isBackgroundProcess = false;
        } 

        if (options.nonce === undefined || options.nonce === "")
        {
            this.getNonce(options);
            return;
        }

        // Create SHA256 password hash with nonce
        var hashedPassword = GetHash(options.password, options.nonce);
        
        // Abort pending requests
        if (loginAjaxCall && loginAjaxCall.isLoading())
        {
            loginAjaxCall.abort();
            loginAjaxCall = null;
            logger.warn("LoginController - doLogin - Pending request aborted");
        }

        // Init login request
        loginAjaxCall = new Ext.data.Connection(
        {
            url: webCallUrl['Login'],
            timeout: 4000
        });

		loginAjaxCall.request(
		{
            method: 'get',			
	        params:
			{
				u: options.username,
				p: hashedPassword,
				s: options.serverId,
				v: appInfoData['appInfo_version'],
                l: currentLanguage
	        },
            success: function(r)
            {
                logger.log("LoginController - doLogin - Success");

                // Is it a background relogin?
                if (options.isRelogin)
                {
                    IpPbxMobile.controllers.UserController.getUserData(options);
                }
                else
                {                
                    // It is a normal login via login form
                    IpPbxMobile.controllers.LoginController.normalLoginSucceeded(options);
                }
            },	        
	        failure: function(r)
			{
                // Is it a background relogin?
                if (options.isRelogin)
                {
                    options.reloginCounter++;
	        	
	        	    Ext.dispatch(
    			    {
    			        controller: IpPbxMobile.controllers.ErrorController,
    			        action: 'reloginFailed',
    			        sourceController: options.sourceController,
    			        sourceAction: options.sourceAction,
    			        isBackgroundProcess: options.isBackgroundProcess,
    			        reloginCounter: options.reloginCounter,
    			        statusCode: r.status,
                        statusText: r.statusText
    			    });

                    return;
                }
                
                // It is a normal login via login form
                Ext.dispatch(
			    {
				    controller: IpPbxMobile.controllers.ErrorController,
				    action: 'requestFailed',
				    sourceController: IpPbxMobile.controllers.LoginController,
				    sourceAction: 'doLogin',
				    statusCode: r.status,
                    statusText: r.statusText
			    });    
	        }
	    });
    },

    //----------------------------------------------------------------
  
    // Receive a new nonce from the IpPbx DataCollector
    getNonce: function(options)
    {
    	logger.log("LoginController - getNonce");

        // Nonce variable defined?
        if (options.nonce === undefined)
        {
            options.proto.nonce = ""; 
        }

        // Abort pending requests
        if (loginAjaxCall && loginAjaxCall.isLoading())
        {
            loginAjaxCall.abort();
            loginAjaxCall = null;
            logger.warn("LoginController - getNonce - Pending request aborted");
        }

        // Init Ajax Call
        loginAjaxCall = new Ext.data.Connection(
        {
            url: webCallUrl['Nonce'],
            timeout: 4000
        });

		loginAjaxCall.request(
		{
            method: 'get',
            success: function(r)
            {
                logger.log("LoginController - getNonce - Success");

                options.nonce = r.responseText.replace(/"/gi, "");
                IpPbxMobile.controllers.LoginController.doLogin(options);
            },	        
	        failure: function(r)
			{
	        	Ext.dispatch(
				{
					controller: IpPbxMobile.controllers.ErrorController,
					action: 'requestFailed',
					sourceController: IpPbxMobile.controllers.LoginController,
					sourceAction: 'getNonce',
					statusCode: r.status,
                    statusText: r.statusText
				});
	        }
	    });
    },
    
    //----------------------------------------------------------------    

    normalLoginSucceeded: function(options)
    {
        // Get new login credentials from login form if auto login is deactivated
        if (!options.useAutoLogin)
        {
            configData.username = options.username;
            configData.password = options.password;
            configData.serverId = options.serverId;
        }

        // Get matching Mobile Extension Number for current user
        if (configData.memNumber === '')
        {
            configData.memNumber = this.getMemNumber(configData.serverId);
        }

        // Save the new login credentials and/or the new MEM number
        IpPbxMobile.controllers.ConfigurationController.save();

        // Hide login loading mask
        if (!isLoggedOn)
        {
            loginMask.hide();
        }
		        
        // Proceed with app startup
	    IpPbxMobile.controllers.StartupController.loadAppData();
    },

    //----------------------------------------------------------------
    
    saveServerList: function(data)
	{
		for (var i=0; i < data.length; i++)
		{		
			var serverDataItem = Ext.ModelMgr.create(
			{
			    id: i,
			    text: data[i].Name,
			    value: i,
			    address: '',
			    memNumber: data[i].MemNumber
			}, 'IpPbxMobile.models.ServerListItem');
			
			serverData.push(serverDataItem);
		}
	},

    //----------------------------------------------------------------

    getMemNumber: function(serverId)
	{
		for (var i=0; i < serverData.length; i++)
		{		
			if (serverData[i].data.id === serverId)
            {
                return serverData[i].data.memNumber;
            }
		}

        return "";
	} 
    
    //----------------------------------------------------------------   
  
});
