//-------------------------------------------------------------------------
// STARTUP CONTROLLER
//-------------------------------------------------------------------------

IpPbxMobile.controllers.StartupController = new Ext.Controller(
{
	// APPLICATION START
	start: function()
    {
        // Create and initialise logger
        logger = new IpPbxMobile.controllers.LoggingController;
        
        // Load Configuration
    	IpPbxMobile.controllers.ConfigurationController.load();

        // Load Application version information from XML file
        this.loadAppInfo();

        // Load Language
		IpPbxMobile.controllers.LanguageController.load();

        // Translate the start pages (URL Page and Login Page)
        IpPbxMobile.controllers.LanguageController.translateStartViews();
        
        // Create loading mask for login process
        loginMask = new Ext.LoadMask(Ext.getBody(), {msg: ''});

        if (configData.url == '' || configData.protocol == '')		
		{
			this.showUrlView();
		}        
		else
		{
            // Show View with logo
            this.showLoadingView();

            // Show login loading mask
    	    loginMask.msg = langData['msg_connecting'];
	        loginMask.show();

            // Generate WebCall URLs
    	    IpPbxMobile.controllers.ConfigurationController.setWebCallUrls();
            
            // Check saved server URL then logs on automatically      
            // If check fails the url view will be displayed      
            // If login fails the login view will be displayed
    	    IpPbxMobile.controllers.LoginController.connect();
		}		
    },

    //----------------------------------------------------------------

    loadAppInfo: function()
    {
        logger.log('StartupController - loadAppInfo');

        var appInfoUrl = 'app/xml/appInfo.xml';

        var xmlhttp = new XMLHttpRequest();
        xmlhttp.open("GET", appInfoUrl, false);
        xmlhttp.setRequestHeader('Content-Type', 'text/xml');
        xmlhttp.send("");
        xmlDoc = xmlhttp.responseXML;

        var appInfoNode = xmlDoc.getElementsByTagName("appInfo");
        if(appInfoNode.length > 0)
        {
            var node = appInfoNode[0];

            var versionNode = node.getElementsByTagName("version");
            if(versionNode && versionNode.length > 0)
            {
                appInfoData['appInfo_version']  = versionNode[0].textContent;
            }
            else
            {
                // Fallback
                logger.warn('appInfo load error: Using fallback for appInfo_version');
                appInfoData['appInfo_version']  = '8.1.0.0';
            }
        }
    },
	
    //----------------------------------------------------------------

    showLoadingView: function()
    {
        logger.log('StartupController - showLoadingView');
		
        // Create start view object
		if (startView === null)
		{
			startView = new IpPbxMobile.views.StartView();
		}

		startView.show(false);

        // Hide button
        startViewButton.hide(false);

        // Hide URL fieldset
        if (!startViewUrlFieldset.hidden)
		{
            startViewUrlFieldset.hide(false);          
		}

        // Hide Login fieldset
        if (!startViewLoginFieldset.hidden)
		{
            startViewLoginFieldset.hide(false);          
		}
    },

    //----------------------------------------------------------------
    
	showUrlView: function()
    {
		logger.log('StartupController - showUrlView');
		
        // Show URL fieldset and hide Login fieldset
        if (startViewUrlFieldset.hidden)
		{
            startViewLoginFieldset.hide(false);
			startViewUrlFieldset.show(false);            
		}

        // Show button and set button caption
        if (startViewButton.hidden)
		{
            startViewButton.show(false);
        }        
        startViewButton.setText(langData['start_btn_connect']);
        
        // Create start view object
		if (startView === null)
		{
			startView = new IpPbxMobile.views.StartView();
		}
	 	
		startView.show(false);

        // Load control content 
		startViewForm.load(IpPbxMobile.stores.ConfigItems.getAt(0));
    },
    
    //----------------------------------------------------------------
    
	showLoginView: function()
    {
		logger.log('StartupController - showLoginView');
		
        // Show Login fieldset and hide URL fieldset
        if (startViewLoginFieldset.hidden)
		{
            startViewUrlFieldset.hide(false);
			startViewLoginFieldset.show(false);            
		}

        // Show button and set button caption
        if (startViewButton.hidden)
		{
            startViewButton.show(false);
        }  
        startViewButton.setText(langData['start_btn_login']);
		
        // Create start view object
		if (startView === null)
		{
			startView = new IpPbxMobile.views.StartView();
		}
		
        startView.show(false);

        // Load control content 
        startViewServerBox.setOptions(serverData, false);        		
		startViewForm.load(IpPbxMobile.stores.ConfigItems.getAt(0));
    },
    
    //----------------------------------------------------------------
    
    // Preload data
	loadAppData: function()
    {
		logger.log('StartupController - loadAppData');

        try
        {
		    // Hide login loading mask
            loginMask.hide();
            
            // Check if this method was called for the first time
		    if (nextLoginStep == null)
		    {
			    nextLoginStep = 'UserData';
		    }

            logger.log('StartupController - loadAppData - ' + nextLoginStep);
		
            // Begin with the application start process
		    switch (nextLoginStep)
		    {
			    case 'UserData':
				    loginMask.msg = langData['msg_load_userdata'];
				    loginMask.show();
				    nextLoginStep = 'LoginStateActions';
				    IpPbxMobile.controllers.UserController.getUserData();
				    break;

                case 'LoginStateActions':
                    loginMask.msg = langData['msg_load_stateactions'];
			        loginMask.show();
                    nextLoginStep = 'Forwardings';
                    this.doLoginStateActions();
				    break;

			    case 'Forwardings':
				    loginMask.msg = langData['msg_load_forwardings'];
				    loginMask.show();
				    nextLoginStep = 'RemoteContacts';
				    IpPbxMobile.controllers.ForwardingController.loadForwardings(false);
				    break;                
				
			    case 'RemoteContacts':
				    loginMask.msg = langData['msg_load_server_contacts'];
				    loginMask.show();
				    nextLoginStep = 'LocalContacts';
				    IpPbxMobile.controllers.RemoteContactController.loadRemoteContacts();
				    break;
				
			    case 'LocalContacts':
				    loginMask.msg = langData['msg_load_phone_contacts'];
				    loginMask.show();
				    nextLoginStep = 'CallJournal';
				    IpPbxMobile.controllers.LocalContactController.loadContacts();
				    break;
				
			    case 'CallJournal':		
				    loginMask.msg = langData['msg_load_journal'];
				    loginMask.show();
				    nextLoginStep = 'PresenceData';
				    IpPbxMobile.controllers.JournalController.loadJournal(false);
				    break;
                    				
                case 'PresenceData':
				    loginMask.msg = langData['msg_load_userdata'];
				    loginMask.show();
				    nextLoginStep = 'ShowMainGui';
				    IpPbxMobile.controllers.UserController.getPresenceData(false);
				    break;

			    case 'ShowMainGui':
				    nextLoginStep = null;
				    isLoggedOn = true;
                    if (!startViewForm.hidden)
		            {
				        startViewForm.hide(false);
                    }
				    IpPbxMobile.showMainGui();
				    break;
		    }
        }
        catch(exc)
        {
            logger.error(exc);
        }
    },

    //----------------------------------------------------------------    

    doLoginStateActions: function()
    {
        logger.log('ConfigurationController - doLoginStateActions');
		
        // If this is the first login use the default settings
        // from the DataCollector and set the user state "Available"
        if (configData.firstLogin)
        {
            configData.postDialableDigits = userData.Defaults.MemDigits;
            configData.useFastDial = userData.Defaults.FastDialup;
            configData.firstLogin = false;
            
            IpPbxMobile.controllers.ConfigurationController.save();
            IpPbxMobile.controllers.UserController.registerDevice();
            return;
        }

        // Restore last mobile device state
        if (configData.mobileDeviceRegState === 2)
        {
            IpPbxMobile.controllers.UserController.registerDevice();
            return;
        }

        this.loadAppData();
    }

    //----------------------------------------------------------------    
});