//-------------------------------------------------------------------------
// USER CONTROLLER
//-------------------------------------------------------------------------

var userAjaxCall = null;

IpPbxMobile.controllers.UserController = new Ext.Controller(
{
	// ---------------------------------------------------------------------

	// Sends device information and receives IpPbx user information
	getUserData : function(options)
    {
		logger.log("UserController - getUserData");
                
        // Check if all required parameters are defined
        if (options === undefined)
        {
            options = { isRelogin: false };
        }

        if (options.isRelogin === undefined)
        {        
            options.proto.isRelogin = false;
        }        

		// Abort pending requests
        if (userAjaxCall && userAjaxCall.isLoading())
        {
            userAjaxCall.abort();
            userAjaxCall = null;
            logger.warn("UserController - getUserData - Pending request aborted");
        }
        
		userAjaxCall = new Ext.data.Connection();
		userAjaxCall.request(
        {
			url: webCallUrl['UserData'],
			method: 'get',
			params:
            {
				a: configData.deviceAppId,
				t: getTimezoneOffset(),
				n: deviceInfo.name,
				o: deviceInfo.platform,
				v: deviceInfo.version
			},
			success : function(r)
            {
			    logger.log("UserController - getUserData - Success");

				userData = JSON.parse(r.responseText);
				
                // Check Help Link
                if (userData.HelpLink == '' &&
                    langData['app_help_link'] != '')
                {
                    userData.HelpLink = langData['app_help_link'];
                }

                // Modify More List Store
                if (userData.HelpLink != '')
                {
                    IpPbxMobile.stores.MoreListItems.getAt(2).data.hidden = false;
                }

                // Save mobile application id
                if (configData.deviceAppId === undefined || configData.deviceAppId === "")
                {
                    configData.deviceAppId = userData.DeviceData.DeviceAppId;
                    IpPbxMobile.controllers.ConfigurationController.save();
                }

                // Check if login is in progress
				if (!isLoggedOn)
                {
                    IpPbxMobile.controllers.StartupController.loadAppData();
				}

                // Check if this is part of a relogin process
                if (options.isRelogin)
                {
                    Ext.dispatch(
				    {
				        controller: options.sourceController,
				        action: options.sourceAction
				    }); 
                }
			},
			failure : function(r)
            {
				logger.log("UserController - getUserData - Failed");

                // Is it a background relogin?
                if (options.isRelogin)
                {
                    Ext.dispatch(
    			    {
    			        controller: IpPbxMobile.controllers.ErrorController,
    			        action: 'reloginFailed',
    			        sourceController: options.sourceController,
    			        sourceAction: options.sourceAction,
    			        isBackgroundProcess: options.isRelogin, // Yes this is correct - isBackgroundProcess = Relogin
    			        reloginCounter: 5,
    			        statusCode: r.status,
                        statusText: r.statusText
    			    });
                }
                else
                {
				    Ext.dispatch(
                    {
					    controller : IpPbxMobile.controllers.ErrorController,
					    action : 'requestFailed',
					    sourceController : IpPbxMobile.controllers.UserController,
					    sourceAction : 'getUserData',					
					    statusCode : r.status,
					    statusText : r.statusText,
                        isBackgroundProcess: options.isRelogin // Yes this is correct - isBackgroundProcess = Relogin
				    });
                }
			}
		});
	},

	// ---------------------------------------------------------------------

	// Get IpPbx user presence information
	getPresenceData : function(isBackgroundProcess)
    {
		logger.log("UserController - getPresenceData");
        
		// Abort pending requests
        if (userAjaxCall && userAjaxCall.isLoading())
        {
            userAjaxCall.abort();
            userAjaxCall = null;
            logger.warn("UserController - getPresenceData - Pending request aborted");
        }

		userAjaxCall = new Ext.data.Connection();
		userAjaxCall.request(
        {
			url : webCallUrl['PresenceData'],
			method : 'get',
            params:
            {
				h: userPresenceData.PictureHash
			},
			success : function(r)
            {
				logger.log("UserController - getPresenceData - Success");

                // Save old presence data
                var oldUserPresenceData = userPresenceData;

                // Get new data from response
                userPresenceData = JSON.parse(r.responseText);
                
				// Check if user picture is still the same
    	        if (userPresenceData.PictureHash == oldUserPresenceData.PictureHash)
        	    {
            	    userPresenceData.PictureData = oldUserPresenceData.PictureData;
                }

	            // If the new user state is "Speaking" the
                // previous state must be used instead
    	        if (userPresenceData.StateId == 2)
        	    {     
            	    if (userPresenceData.IsAway)
                        userPresenceData.StateId = 4;
                    else if (userPresenceData.IsDND)
                        userPresenceData.StateId = 5;
                    else userPresenceData.StateId = 1;
                }
                
                // Update My Profile Page
                IpPbxMobile.controllers.UserController.updateProfileView(false);

                // Special handling for login process
                if (!isLoggedOn)
                {
                    IpPbxMobile.controllers.StartupController.loadAppData();
                    return;
				}
			},
			failure : function(r)
            {
				logger.log("UserController - getPresenceData - Failed");

				Ext.dispatch(
                {
					controller : IpPbxMobile.controllers.ErrorController,
					action : 'requestFailed',
					sourceController : IpPbxMobile.controllers.UserController,
					sourceAction : 'getPresenceData',
					isBackgroundProcess : isBackgroundProcess,
					statusCode : r.status,
					statusText : r.statusText
				});
			}
		});
	},
            
	// ---------------------------------------------------------------------

	setUserState : function(stateId)
    {
		logger.log("UserController - setUserState: " + stateId);
		
        // Check if all required parameters are defined
        if (stateId === undefined ||
		    stateId === null ||
			stateId < 0 ||
			stateId > 5)
        {        
			logger.warn("UserController - setUserState - Invalid state");
            return false;
        }
        
        // Display loading screen
        var setStateMask = new Ext.LoadMask(Ext.getBody(), {msg: langData['msg_saving']});
  	    setStateMask.show();

        // Abort pending requests
        if (userAjaxCall && userAjaxCall.isLoading())
        {
            userAjaxCall.abort();
            userAjaxCall = null;
            logger.warn("UserController - setUserState - Pending request aborted");
        }

        // Init Ajax call        
		userAjaxCall = new Ext.data.Connection();
		userAjaxCall.request(
        {
			url: webCallUrl['SetUserState'],
			method: 'get',
			params:
            {
				s : stateId
			},
			success : function(r)
            {
				logger.log("UserController - setUserState - Success");
                
                // Save User State
                userPresenceData.StateId = stateId;

                // Set device registration state
                if (userPresenceData.StateId == 0)
                    configData.mobileDeviceRegState = 0;
                else configData.mobileDeviceRegState = 2;

                IpPbxMobile.controllers.ConfigurationController.save();

                // Update My Profile Page
                IpPbxMobile.controllers.UserController.updateProfileView(true);

                // Hide loading mask
                setStateMask.destroy();
                setStateMask = null;

                // Show My Profile Page if method was called as result of an GUI interaction                
                IpPbxMobile.views.userCard.setActiveItem(0);
			},
			failure : function(r)
            {
				logger.warn("UserController - setUserState - Failed");
                
                // Hide loading mask
                setStateMask.destroy();
                setStateMask = null;

                Ext.dispatch(
                {
					controller : IpPbxMobile.controllers.ErrorController,
					action : 'requestFailed',
					sourceController : IpPbxMobile.controllers.UserController,
					sourceAction : 'doUserStateChange',
					statusCode : r.status,
				    statusText : r.statusText
				});
			}
		});
	},

    // ---------------------------------------------------------------------

	// ONLY CALL THIS METHOD ON APPLICATION STARTUP
	registerDevice : function()
    {
		logger.log("UserController - registerDevice");
		
        // Abort pending requests
        if (userAjaxCall && userAjaxCall.isLoading())
        {
            userAjaxCall.abort();
            userAjaxCall = null;
            logger.warn("UserController - registerDevice - Pending request aborted");
        }

        // Init Ajax call 
		userAjaxCall = new Ext.data.Connection();
		userAjaxCall.request(
        {
			url: webCallUrl['RegisterDevice'],
			method: 'get',			
            scope: this,
			success : function(r)
            {
				logger.log("UserController - registerDevice - Success");

                // Set device registration state
                configData.mobileDeviceRegState = 1; // New fresh registration
                IpPbxMobile.controllers.ConfigurationController.save();

                // If this method was called during login return to
                // the "loadAppData" method in the Startup Controller
                if (!isLoggedOn)
                {                   
					IpPbxMobile.controllers.StartupController.loadAppData();
                    return;
				}
			},
			failure : function(r)
            {
				logger.warn("UserController - registerDevice - Failed");
                
                Ext.dispatch(
                {
					controller : IpPbxMobile.controllers.ErrorController,
					action : 'requestFailed',
					sourceController : IpPbxMobile.controllers.UserController,
					sourceAction : 'registerDevice',
					statusCode : r.status,
				    statusText : r.statusText
				});
			}
		});
	},

    // ---------------------------------------------------------------------

	// ONLY CALL THIS METHOD ON FACTORY RESET OR USER SWITCH
    unregisterDevice : function()
    {
		logger.log("UserController - unregisterDevice");
		
        // Abort pending requests
        if (userAjaxCall && userAjaxCall.isLoading())
        {
            userAjaxCall.abort();
            userAjaxCall = null;
            logger.warn("UserController - unregisterDevice - Pending request aborted");
        }

        // Init Ajax call 
		userAjaxCall = new Ext.data.Connection();
		userAjaxCall.request(
        {
			url: webCallUrl['UnregisterDevice'],
			method: 'get',			
            scope: this,
			success : function(r)
            {
				logger.log("UserController - unregisterDevice - Success");

                // Set device registration state
                configData.mobileDeviceRegState = 0;
                IpPbxMobile.controllers.ConfigurationController.save();
			},
			failure : function(r)
            {
				logger.warn("UserController - unregisterDevice - Failed");
              
                Ext.dispatch(
                {
					controller : IpPbxMobile.controllers.ErrorController,
					action : 'requestFailed',
					sourceController : IpPbxMobile.controllers.UserController,
					sourceAction : 'registerDevice',
					statusCode : r.status,
				    statusText : r.statusText,
                    isBackgroundProcess : true
				});
			}
		});
	},

    // ---------------------------------------------------------------------
	// Updates the status text on server (if needed)

    updateUserFreeText: function(options)
    {
        var statusText;
        if (Ext.isObject(options) &&
            options.statusText !== undefined)
        {
            statusText = options.statusText;
        }
        else
        {
            statusText = profileUserStatusText.getValue();
        }

		var currentText = IpPbxMobile.controllers.LanguageController.getLocalisedStatusText(
                            userPresenceData.CurrentText);

        if (statusText !== currentText)
        {
            this.setUserFreeText({ statusText: statusText });
        }
    },

	// ---------------------------------------------------------------------

	// Set IpPbx user free status text
	setUserFreeText : function(options)
    {
		logger.log("UserController - setUserFreeText");

		// Abort pending requests
        if (userAjaxCall && userAjaxCall.isLoading())
        {
            userAjaxCall.abort();
            userAjaxCall = null;
            logger.warn("UserController - setUserFreeText - Pending request aborted");
        }

        // Init Ajax call 
		userAjaxCall = new Ext.data.Connection();
		userAjaxCall.request(
        {
			url: webCallUrl['SetUserFreeText'],
			method: 'get',
			params:
            {
				t : options.statusText
			},
			success: function(r)
            {
				logger.log("UserController - setUserFreeText - Success");

                // Update last profile change date
                lastProfileChange = getUnixTimestamp();
                IpPbxMobile.controllers.ConfigurationController.save();

                userPresenceData.CurrentText = this.params.t;
			},
			failure: function(r)
            {
				logger.warn("UserController - setUserFreeText - Failed");

				Ext.dispatch(
                {
					controller : IpPbxMobile.controllers.ErrorController,
					action : 'requestFailed',
					sourceController : IpPbxMobile.controllers.UserController,
					sourceAction : 'setUserFreeText',
					statusCode : r.status,
					statusText : r.statusText
				});
			}
		});
	},

    // ---------------------------------------------------------------------
   
    resetUserStatusText: function()
    {
        logger.log("UserController - resetUserStatusText");

        var statusText = IpPbxMobile.controllers.LanguageController.getLocalisedStatusText(userPresenceData.CurrentText);
        profileUserStatusText.setValue(statusText);    
    },

    //---------------------------------------------------------------------
    // GUI INTERACTION
    //---------------------------------------------------------------------

    doUserSwitch: function()
    {
        logger.log("UserController - doUserSwitch");

        // Show Loading Mask
        var userSwitchMask = new Ext.LoadMask(Ext.getBody(), {msg: langData['user_card_msg_switch_user'] });
		userSwitchMask.show();

        // Remove mobile device registration
        IpPbxMobile.controllers.UserController.unregisterDevice();

        // Reset login globals
        isLoggedOn = false;
        nextLoginStep = null;

        // Modify Configuration Data
		IpPbxMobile.controllers.ConfigurationController.resetConfigForUserSwitch();

        // Do Application Restart
        var restartTask = new Ext.util.DelayedTask(function()
        {
            location.reload();
        });

        restartTask.delay(5000);
    },

    // ---------------------------------------------------------------------
   
    showUserStateChangePage: function()
    {
        logger.log("UserController - showUserStateChangePage");

        this.setUserStateChangePageContent();

       	IpPbxMobile.views.userCard.setActiveItem(1);
    },   

    // ---------------------------------------------------------------------

    setUserStateChangePageContent: function()
    {
        logger.log("UserController - setUserStateChangePageContent");

        // If device not registered select user state "Logged off" (alias "Not visible")
        if (configData.mobileDeviceRegState == 0)
        {
            IpPbxMobile.controllers.UserController.setCurrentState(0);
        }
        else IpPbxMobile.controllers.UserController.setCurrentState(configData.currentUserStateId);            
    },   

    // ---------------------------------------------------------------------

    doUserStateChange: function()
    {
        logger.log("UserController - doUserStateChange");

        // Check if the user changed his state 
        var newUserStateId = IpPbxMobile.controllers.UserController.getCurrentState();

        if (newUserStateId != configData.currentUserStateId)
        {
            logger.log("New user state selected: " + newUserStateId);
            IpPbxMobile.controllers.UserController.setUserState(newUserStateId);
        }
        else
        {        
            logger.log("User state NOT changed");
            IpPbxMobile.views.userCard.setActiveItem(0);
        }
    },

    //---------------------------------------------------------------------

    setCurrentState: function(stateId)
    {
        switch (stateId)
        {
            case 0:
                IpPbxMobile.views.userCard.query('#userStateNotVisible')[0].setChecked(true);
                break;
            case 1:
                IpPbxMobile.views.userCard.query('#userStateAvailable')[0].setChecked(true);
                break;
            case 4:
                IpPbxMobile.views.userCard.query('#userStateAway')[0].setChecked(true);
                break;
            case 5:
                IpPbxMobile.views.userCard.query('#userStateDND')[0].setChecked(true);
                break;
        }
    },
    
    //---------------------------------------------------------------------

    getCurrentState: function()
    {
        if (IpPbxMobile.views.userCard.query('#userStateNotVisible')[0].isChecked())
            return 0;

        if (IpPbxMobile.views.userCard.query('#userStateAvailable')[0].isChecked())
            return 1;

        if (IpPbxMobile.views.userCard.query('#userStateAway')[0].isChecked())
            return 4;

        if (IpPbxMobile.views.userCard.query('#userStateDND')[0].isChecked())
            return 5;
    },
        
    //---------------------------------------------------------------------
    // PROFILE PAGE - GUI UPDATE
    //---------------------------------------------------------------------

    updateProfileView : function(statusChangedManually)
    { 
        // Manual status change via MobileApp
        if (statusChangedManually)
        {
            lastProfileChange = getUnixTimestamp();
        }
        
        // Save new user state if mobile device is registered
        if (configData.mobileDeviceRegState > 0)        
            configData.currentUserStateId = userPresenceData.StateId;
        else configData.currentUserStateId = 0;
        
        // If user state is not "Logged Off" the device
        // registration will be marked as confirmed
        if (configData.currentUserStateId > 0 &&
            configData.mobileDeviceRegState == 1)
        {
            configData.mobileDeviceRegState = 2;
        }
        
        // Check if user state matches with device registration state

        // No device is registered for this user
        // and the MobileApp logs on the first time
        // Result: The DataCollector will need some
        // time to get the real new user from the CDS
        // Therefore we must calculate the correct state
        if (configData.currentUserStateId == 0 &&
            configData.mobileDeviceRegState == 1)
        {
            if (userPresenceData.IsAway)
                configData.currentUserStateId = 4;
            else if (userPresenceData.IsDND)
                configData.currentUserStateId = 5;
            else configData.currentUserStateId = 1;
        }

        // Check if mobile device registration is still valid

        // IpPbx Services were restarted OR
        // All devices of this user were kicked
        // via AdminConsole (forced logoff)
        // Result: The MobileApp is not registered anymore
        if (configData.currentUserStateId == 0 &&
            configData.mobileDeviceRegState == 2)
        {
            configData.mobileDeviceRegState = 0;           
        }        

        // Save Config Data
        IpPbxMobile.controllers.ConfigurationController.save();
        
        // GUI UPDATE

        // Check if username has been changed
        if (profileUserName.getTargetEl() == undefined ||
			profileUserName.getTargetEl().getHTML() != userData.Username)
        {
            // Update Username
            profileUserName.update(userData.Username);
        }


        // Check if User Forwarding Status must be updated
        this.updateProfileViewState();


        // Check if Status Text must be updated
        var newStatusText = IpPbxMobile.controllers.LanguageController.getLocalisedStatusText(userPresenceData.CurrentText);
        if (newStatusText != profileUserStatusText.getValue())
        {
            profileUserStatusText.setValue(newStatusText);
        }

        
        // Check if User Status must be updated
        this.updateProfileViewState();
        
       
         // Check if User Picture has to be updated
        var picture = '<div class="detailPicture defaultUserPic"></div>';
        if (userPresenceData.PictureData != '' && userPresenceData.PictureData != null)
        {
            picture = '<div class="detailPicture" style="background-image:url(\'data:image/png;base64,' + userPresenceData.PictureData + '\');"></div>';
		}
		
        if (profileUserPicture.getTargetEl() == undefined ||
            picture != profileUserPicture.getTargetEl().getHTML())
        {
            profileUserPicture.update(picture);
        }
	},

    // -------------------------------------------------------------------------
	
    updateProfileViewFwdState: function()
    {
        // Check if Forwarding State Area has to be shown or hidden
        if (userPresenceData.Fwd.length == 0 && !profileForwardingState.hidden)
        {   
            profileForwardingState.hide(false);
        }
        else if (userPresenceData.Fwd.length > 0 && profileForwardingState.hidden)
        {        
            profileForwardingState.show();
        }

        // Check if Forwarding State Area Content must be updated
        var currentFwdText = null;
        
        if (profileForwardingState.getTargetEl() != undefined)
        {
            currentFwdText = profileForwardingState.getTargetEl().getHTML();
        }
            
        if (userPresenceData.Fwd == "vm")
        {   
            if (currentFwdText != langData['user_card_fwd_active_vm'])
                profileForwardingState.update(langData['user_card_fwd_active_vm']);
        }        
        else if (userPresenceData.Fwd == "on")
        {
            if (currentFwdText != langData['user_card_fwd_active_number'])
                profileForwardingState.update(langData['user_card_fwd_active_number']);
        }
        else if (userPresenceData.Fwd.length > 0)
        {
            var fwdText = langData['user_card_fwd_active_number_ex'];
            fwdText = fwdText.replace(/\{0\}/gi, userPresenceData.Fwd);
            profileForwardingState.update(fwdText);
        }                
    },

    // -------------------------------------------------------------------------
	
    updateProfileViewState: function()
    {
        // Get State Id

        // 0 = Logged Off
		// 1 = Available
		// 2 = Speaking
		// 3 = No state available
		// 4 = Away
		// 5 = Do not disturb

        // Check if Status Name must be updated
        var newStateName = IpPbxMobile.controllers.LanguageController.getLocalisedStatusName(configData.currentUserStateId, true);
        if(profileUserStatus.getTargetEl() === undefined ||
		   newStateName != profileUserStatus.getTargetEl().getHTML())
        {
            profileUserStatus.update(newStateName);
        }

        // Build 'div' for user status icon
        var statusIcon = '<div class="profileStatusIcon cState';
        
        if (configData.currentUserStateId == 0)
            statusIcon += '3';
        else statusIcon += configData.currentUserStateId;

        statusIcon += '"></div>';
         
        // Check if User State Icon has to be updated
        if (profileUserStatusIcon.getTargetEl() == undefined ||
           (statusIcon != profileUserStatusIcon.getTargetEl().getHTML()))
        {
            profileUserStatusIcon.update(statusIcon);
        }
    },

    //------------------------------------------------------------------------
	// PROFILE PAGE - CAPTURE USER PICTURE
    //------------------------------------------------------------------------
	
    captureUserPicture: function()
    {
        // App running on device?
        if (navigator.camera === undefined)
        {
            logger.warn("UserController - captureUserPicture - No physical device found");
            return;
        }

        navigator.camera.getPicture(IpPbxMobile.controllers.UserController.onCaptureSuccess,
                                    IpPbxMobile.controllers.UserController.onCaptureFailure,
        {
            quality : 75, 
            destinationType : Camera.DestinationType.DATA_URL, 
            sourceType : Camera.PictureSourceType.CAMERA, 
            allowEdit : true,
            encodingType: Camera.EncodingType.PNG,
            targetWidth: 100,
            targetHeight: 100
        });
    },

    //------------------------------------------------------------------------
    
    onCaptureSuccess: function(imageData)
    {
        // Check if captured image is valid
        if (imageData == null ||
            imageData == '')
        {            
            new Ext.ux.ResultMask(Ext.getBody(),
		    {
		        msg: langData['user_card_msg_change_userpic_invalid'],
		        type: 'error',
		        timeout: 5000
		    });

            return;
        }

        // Ask user if he really want to save this picture
        var msgBox = new Ext.MessageBox;
        msgBox.confirm(langData['cn_confirmation'], langData['user_card_msg_change_userpic_ask'], function(e)
        {
            msgBox.hide(false);
            
            if (e == "yes")
            {                
                IpPbxMobile.controllers.UserController.sendUserPicture(imageData);
                return;
            }
        });
    },

    //------------------------------------------------------------------------

    onCaptureFailure: function(errorMsg)
    {
        var message = langData['user_card_msg_change_userpic_failed'];

        if (errorMsg != null &&
            errorMsg != '')
        {
            message += ' (' + errorMsg + ')';
        }

        new Ext.ux.ResultMask(Ext.getBody(),
		{
		    msg: message,
		    type: 'error',
		    timeout: 5000
		});
    },

    //------------------------------------------------------------------------

    sendUserPicture: function(pictureData)
    {
        logger.log("UserController - sendUserPicture");
		
        // Display saving screen
        userPictureMask = new Ext.LoadMask(Ext.getBody(), {msg: langData['msg_saving']});
  	    userPictureMask.show();

        // Abort pending requests
        if (userAjaxCall && userAjaxCall.isLoading())
        {
            userAjaxCall.abort();
            userAjaxCall = null;
            logger.warn("UserController - sendUserPicture - Pending request aborted");
        }

        // Init Ajax Call
        userAjaxCall = new Ext.data.Connection(
        {
            url: webCallUrl['SetUserPicture'],
            timeout: 20000
        });

        userAjaxCall.request(
		{
		    method: 'post',
            headers:
            {
                'Content-Type': 'application/json;charset=utf-8'
            },
		    params: Ext.encode(pictureData),
            scope: this,
			success : function(r)
            {
				logger.log("UserController - sendUserPicture - Success");

                // Hide saving mask
                userPictureMask.hide();

                // Show loading mask
                userPictureMask = new Ext.LoadMask(Ext.getBody(), {msg: langData['msg_loading']});
                userPictureMask.show();

                var refreshProfilePageTask = new Ext.util.DelayedTask(function()
                {                    
                    IpPbxMobile.controllers.UserController.getPresenceData(true);
                    userPictureMask.hide();
                });

                refreshProfilePageTask.delay(3000);
			},
			failure : function(r)
            {
				logger.warn("UserController - sendUserPicture - Failed");
              
                // Hide saving mask
                userPictureMask.hide();

                Ext.dispatch(
                {
					controller : IpPbxMobile.controllers.ErrorController,
					action : 'requestFailed',
					sourceController : IpPbxMobile.controllers.UserController,
					sourceAction : 'sendUserPicture',
					statusCode : r.status,
				    statusText : r.statusText
				});
			}
		});
    }

    //------------------------------------------------------------------------
});
