//-------------------------------------------------------------------------
// REMOTE CONTACTS - MODEL
//-------------------------------------------------------------------------

Ext.regModel('IpPbxMobile.models.RemoteContactItem',
{
    fields: [
        {name: 'Id', type: 'int' },         // IpPbx User Id (for Phonebook Entries this is -1)
        {name: 'RefId', type: 'int' },      // Unique Phonebook Entry Reference Id
	    {name: 'Name', type: 'string' },    // Name (e.g. Username)
	    {name: 'Mail', type: 'string' },    // Mail Address
	    {name: 'Site', type: 'int' },       // IpPbx Site Id        
	    {name: 'Pic', type: 'string' },     // Base64 Picture Data
        {name: 'PicHash', type: 'string' }, // Picture File Hash
        {name: 'Text', type: 'string' },    // Free Status Text
        {name: 'PbNum', type: 'string' },   // Personal or Editable Phonebook Number	 	
	 	{name: 'Nums', type: 'auto' },      // Array of Internal and Public Numbers
        {name: 'ExNums', type: 'auto' },    // Public Numbers from other sites which
                                            // do not have an assigned internal number

        // Forwarding
        // on = Forwarding is active (number hidden)
        // vm = Forwarding to voicemail is active
        // number (e.g. +4912345) = Forwarding is active (number visible)
        // empty string = Forwarding is not active
        {name: 'Fwd', type: 'string' },
	    {name: 'Forwarding', type: 'string', convert: function (value, record)
            {
	            var state = record.get('Fwd');
	            switch (state)
                {
	                case 'on':
	                    return langData['contacts_fwd_active_number'];
	                case 'vm':
	                    return langData['contacts_fwd_active_vm'];
	                case '':
	                    return state;
	                case undefined:
	                    return '';
	                default:
	                    return langData['contacts_fwd_active_number'] + ' (' + state + ')';
	            }
	        }
	    },

        //State
        // 0 = Logged Off
        // 1 = Available
        // 2 = Speaking
        // 3 = No state available
        // 4 = Away
        // 5 = Do not disturb	 	
	 	{name: 'State', type: 'int' },

        // Type
        // 0 = Global Phonebook Entry
        // 1 = IpPbxUser
        // 2 = Group
        // 3 = Personal Phonebook Entry	 	
	 	{name: 'Type', type: 'int' },


        {name: 'Numbers', type: 'auto', convert: function (value, record)
            {
                var numbers = record.get('Nums');
                var newNumbers = new Array();
                
                if (Ext.isArray(numbers) && numbers.length > 0)
                {
                    var isPublicNumberAvailable = false;

                    for (var i = 0; i < numbers.length; i++)
                    {
                        if (numbers[i].PNums[0] == undefined)
                        {
                            newNumbers.push(numbers[i].INum);
                            continue;
                        }

                        isPublicNumberAvailable = false;

                        for (var j = 0; j < numbers[i].PNums.length; j++)
                        {
                            if (numbers[i].PNums[j].search(/@/) == -1)
                            {                            
                                newNumbers.push(numbers[i].PNums[j]);
                                isPublicNumberAvailable = true;
                            }
                        }

                        if (!isPublicNumberAvailable)
                        {
                            newNumbers.push(numbers[i].INum);
                        }
                    }
                }

                var externalNumbers = record.get('ExNums');

                if (Ext.isArray(externalNumbers) && externalNumbers.length > 0)
                {
                    for (var z = 0; z < externalNumbers.length; z++)
                    {
                        if (externalNumbers[z].search(/@/) == -1)
                        {
                            newNumbers.push(externalNumbers[z]);
                        }
                    }
                }

                var pbNum = record.get('PbNum');

                if (pbNum != undefined && pbNum.length > 0)
                {
                    newNumbers.push(pbNum);
                }

                return newNumbers;
            }
        },

	 	{name: 'TypeName', type: 'string' },
        {name: 'StateName', type: 'string', convert: function (value, record)
            {
                return IpPbxMobile.controllers.LanguageController.getLocalisedStatusName(record.get('State'), false);
            }
        },

        {name: 'LocalisedText', type: 'string', convert: function (value, record)
            {
                var statusText = record.get('Text');

                statusText = statusText.replace("%ab%", langData['user_card_AppointmentBusyText']);
                statusText = statusText.replace("%ao%", langData['user_card_AppointmentOutOfOfficeText']);
                statusText = statusText.replace("%ap%", langData['user_card_AppointmentPrivateText']);

                return statusText;                
            }
        },

        {name: 'id', convert: function (value, record)
            {
                var id = record.get('RefId');
                return id;
            }
        }
	]
});

//-------------------------------------------------------------------------

// TODO: Is the following model still in use?
Ext.regModel('IpPbxMobile.models.RemoteContactDetails',
{
    fields: [
       {name: 'Id', type: 'int' },
       {name: 'Name', type: 'string' },       // IpPbx User Id (for Phonebook Entries this is -1)
       {name: 'Desc', type: 'string' },
       {name: 'id', convert: function (value, record)
           {
               var id = record.get('Id');
               return id;
           }
       }
	],
    proxy:
    {
        type: 'ajax',
        reader:
        {
            type: 'json',
            root: function (data)
            {
                return new Array(data);
            }
        },
        autoLoad: false
    }
});

//-------------------------------------------------------------------------
// LOCAL CONTACTS - MODEL
//-------------------------------------------------------------------------

Ext.regModel('IpPbxMobile.models.LocalContactItem',
{
    fields: [
        {name: 'id', type: 'int' },        
        {name: 'givenName', type: 'string' },    // Firstname
        {name: 'familyName', type: 'string' },   // Lastname
        {name: 'emails', type: 'auto' },         // Array of Mail Addresses
        {name: 'phoneNumbers', type: 'auto'},    // Array of Phone Numbers
        {name: 'organizations', type: 'auto'},   // Array of Companies
        {name: 'photos', type: 'auto'},          // Array of User Pictures (Avatars)
        {name: 'company', type: 'string'},       // Company name
        {name: 'jobTitle', type: 'string'},      // Job Title
        {name: 'displayPhoto', type: 'string'},   // Picture location
        {name: 'displayName', convert:
            function (value, record)
            {
                var givenName = record.get('givenName');
                var familyName = record.get('familyName');

                givenName = Ext.util.Format.trim(givenName);
                familyName = Ext.util.Format.trim(familyName);
               
                if (givenName != "" && familyName != "")
                {
                    return familyName + ", " + givenName;    
                }
                
                if (givenName == "" && familyName != "")
                {
                    return familyName;    
                }

                if (givenName != "" && familyName == "")
                {
                    return givenName;
                }

                // Check company name
                var company = record.get('company');
                if(company && company !== '')
                {
                    return company;
                }

                // If we reach that point, return first (not empty) number
                // as display name
                var numbers = record.get('phoneNumbers');
                if (Ext.isArray(numbers) && numbers.length > 0)
                {
                    for(var i in numbers)
                    {
                        var no = numbers[i].value;
                        if(no !== '')
                            return no;
                    }
                }

                // As a last fallback we just show the EMail address of the entry
                var emails = record.get('emails');
                if (Ext.isArray(emails) && emails.length > 0)
                {
                    for(var i in emails)
                    {
                        var em = emails[i].value;
                        if(em !== '')
                            return em;
                    }
                }

                return "";
            }
        },
        {name: 'sortField', convert:
            function (value, record)
            {
                // Use familyName to sort records
                var dispName = record.get('displayName');
                return dispName.toUpperCase();
            }
        }
    ]
});

//-------------------------------------------------------------------------