//-------------------------------------------------------------------------
// CALL JOURNAL - MODEL
//-------------------------------------------------------------------------

Ext.regModel('IpPbxMobile.models.JournalItem',
{
    fields: [
        { name: 'Id', type: 'int' },               // Call Entry Id
	    {name: 'CallingName', type: 'string' },   // Name of caller   
        {name: 'CallingNumber', type: 'string' }, // Number of caller   
        {name: 'CalledName', type: 'string' },    // Name of callee   
        {name: 'CalledNumber', type: 'string' },  // Number of callee   
        {name: 'Date', type: 'string' },          // Call DateTime value as formatted string
        {name: 'DateTS', type: 'int' },           // Call DateTime value as Unix Timestamp
        {name: 'Viewed', type: 'int' },           // Is it a missed call? 1=Viewed 0=Not viewed
        {name: 'Callback', type: 'int' },         // Was a callback requested?      

        // Call State Id
        // 0 = Accepted
        // 1 = NotAccepted
        // 2 = AskedForCallback
        // 3 = CalledBack
        // 4 = Diverted
        // 5 = DivertedVoicemail
        // 6 = Unknown
        // 7 = AcceptedByTransfer
        {name: 'State', type: 'int' },

        // Type
        // 1 = Caller List (in)
        // 2 = Redial List (out)
	 	{name: 'Type', type: 'int' },

        // Return entry name
        {name: 'EntryName', type: 'string',
            convert: function (value, record)
            {
                var name = '';
                var number = '';

                if (record.get('Type') == 1) // Incoming call
                {
                    name = record.get('CallingName');
                    number = record.get('CallingNumber');
                }
                else if (record.get('Type') == 2) // Outgoing call
                {
                    name = record.get('CalledName');
                    number = record.get('CalledNumber');
                }

                if (name.length > 0)
                    return name;

                if (number.length > 0)
                    return number;

                return langData["journal_call_unknown"];
            }
        },

        // Return entry number
        {name: 'EntryNumber', type: 'string',
            convert: function (value, record)
            {
                if (record.get('Type') == 1) // Incoming call
                {
                    return record.get('CallingNumber');
                }
                else if (record.get('Type') == 2) // Outgoing call
                {
                    return record.get('CalledNumber');
                }

                return '';
            }
        },

        // Is redirected call (e.g. group call)
        {name: 'IsRedirectedCall', type: 'bool',
            convert: function (value, record)
            {
                // Check if this is an incoming call
                if (record.get('Type') != 1)
                    return false;

                var calledName = record.get('CalledName');

                if (calledName && calledName != userData.Username)
                    return true;

                return false;
            }
        },

        // Is missed call
        {name: 'IsMissedCall', type: 'bool',
            convert: function (value, record)
            {
                // Check if this is an outgoing call
                if (record.get('Type') == 2)
                    return false;

                // This must be an incoming call

                var callStateId = record.get('State');

                // Call State Id
                // 0 = Accepted
                // 1 = NotAccepted
                // 2 = AskedForCallback
                // 3 = CalledBack
                // 4 = Diverted
                // 5 = DivertedVoicemail
                // 6 = Unknown
                // 7 = AcceptedByTransfer

                switch (callStateId)
                {
                    case 1:
                    case 2:
                    case 4:
                    case 5:
                        return true;
                }

                return false;
            }
        },

         // Search for matching remote contact item
        {name: 'RemoteContactItem', type: 'RemoteContactItem',
            convert: function (value, record)
            {
                var name = record.get('EntryName');
                var contact = Ext.getStore('RemoteContactItemStore').findRecord('Name', name);
                return contact;
            }
        },

        // Search for matching local contact item
       {name: 'LocalContactItem', type: 'LocalContactItem',
           convert: function (value, record)
           {
                //TODO Find contact by phonenumber
               return null;
           }
       },

        // 0 = Unknown
        // 1 = Remote contact
        // 2 = Local contact
        {name: 'ContactType', type: 'int',
            convert: function (value, record)
            {
                if (record.get('RemoteContactItem') != undefined)
                    return 1;
                if (record.get('LocalContactItem') != undefined)
                    return 2;
                else
                    return 0;
            }
        },

        // Call State Name
        {name: 'StateName', type: 'string',
            convert: function (value, record)
            {
                var state = record.get('State');
                var type = record.get('Type');

                return IpPbxMobile.controllers.LanguageController.getLocalisedCallStateName(type, state);
            }
        },

        // Returns the localized version of the string "Call for"
        {name: 'CallForString', type: 'string',
            convert: function (value, record)
            {
                return langData['journal_call_for'];
            }
        }
    ]
});

