//------------------------------------------------------------------------------------------------
// SETTINGS CARD
//------------------------------------------------------------------------------------------------

IpPbxMobile.views.SettingsCard = Ext.extend(Ext.form.FormPanel,
{
    hidden: true,
    scroll: 'vertical',
    cls: 'sPanel',
    
    initComponent: function()
    {        
        Ext.apply(this,
		{
		    items: [
                {
                    xtype: 'fieldset',
                    centered: true,                    
                    defaults:
                    {
                        listeners:
                        {
                            change: this.onChangeFormValue
                        }
                    },
                    items: [
                    {
                        // Enable Automatic Login (default ON)
                        xtype: 'togglefield',
                        labelWidth: 170,
                        name: 'useAutoLogin',
                        label: 'Auto Login',
                        itemId: 'useAutoLogin'
                    },
                    {
                        // Enable Fast Dial (default ON)
                        xtype: 'togglefield',
                        labelWidth: 140,
                        itemId: 'useFastDial',
                        name: 'useFastDial',
                        label: 'Enable Fast Dial'
                    },
                    {
                        // Dial Mode Selection Box (default ASK)
                        xtype: 'selectfield',
                        name: 'dialMode',
                        label: 'Dial Mode',
                        itemId: 'dialMode',
                        cls: 's-field-select',
                        inputCls: 's-input-text',
                        componentCls: 's-field',
                        hideOnMaskTap: true,
                        showAnimation: false,
                        labelWidth: 140,
                        options: [
	                         { text: 'Ask', value: 0 },
                             { text: 'Office', value: 1 },
                             { text: 'Mobile', value: 2 }
                        ]
                    },
                    {
                        // Max. Post Dialable Digits Selection Box (default 0)
                        xtype: 'selectfield',
                        name: 'postDialableDigits',
                        itemId: 'postDialableDigits',
                        hideOnMaskTap: true,
                        cls: 's-field-select',
                        inputCls: 's-input-text',
                        componentCls: 's-field',
                        showAnimation: false,
                        labelWidth: 200,
                        options: [
	                         { text: '0', value: 0 },
                             { text: '1', value: 1 },
                             { text: '2', value: 2 },
                             { text: '3', value: 3 },
                             { text: '4', value: 4 },
                             { text: '5', value: 5 },
                             { text: '6', value: 6 },
                             { text: '7', value: 7 },
                             { text: '8', value: 8 },
                             { text: '9', value: 9 },
                             { text: '10', value: 10 },
                             { text: '11', value: 11 },
                             { text: '12', value: 12 },
                             { text: '13', value: 13 },
                             { text: '14', value: 14 },
                             { text: '15', value: 15 }
                        ]
                    }]
                },
                {
                    xtype: 'fieldset',
                    centered: true,
                    title: 'MobileExtensionManager',
                    defaults:
                    {
                        listeners:
                        {
                            keyup: this.onChangeFormValue
                        }
                    },
                    items: [
                    {
                        // Field for Mobile Extension Number (readonly)
                        xtype: 'textfield',
                        labelWidth: 80,
                        name: 'memNumber',
                        inputType: 'tel',
                        inputCls: 's-input-tel',
                        componentCls: 's-field',
                        label: 'Number',
                        itemId: 'memNumber'
                    }]
                },
                {
                    xtype: 'fieldset',
                    title: 'Diagnostics',
                    itemId: 'fieldsetDiagnostics',
                    centered: true,
                    defaults:
                    {
                        listeners:
                        {
                            change: this.onChangeFormValue
                        }
                    },
                    items: [
                    {
                        // Enable Logging (default OFF)
                        xtype: 'togglefield',
                        labelWidth: 140,
                        name: 'useLogging',
                        label: 'Logging',
                        itemId: 'useLogging'                    
                    }]
                }]
		});

        IpPbxMobile.views.SettingsCard.superclass.initComponent.apply(this, arguments);
    },

    onChangeFormValue: function (obj, thumb, newValue, oldValue)
    {
		// Togglefield bug: If such a field is rendered
		// the oldValue is "" instead of undefined.
		// This must be handled correctly
        if (obj.xtype === "togglefield" && oldValue === "")
            return;
                                
        IpPbxMobile.controllers.MoreController.saveSettings();
    },

    // Here we overwrite the "loadRecord" method and add
    // a new class attribute "isRecordLoadInProgress".
    // The attribute make it possible to check if a
    // "recordLoad" is in progress.

    isRecordLoadInProgress: false,
    loadRecord: function()
    {
        this.isRecordLoadInProgress = true;        
        IpPbxMobile.views.SettingsCard.superclass.loadRecord.apply(this, arguments);        
        this.isRecordLoadInProgress = false;
    }
});

//Register View
Ext.reg('IpPbxMobile.views.SettingsCard', IpPbxMobile.views.SettingsCard);

//------------------------------------------------------------------------------------------------
// CONTROLS
//------------------------------------------------------------------------------------------------

// Reset Button
var settingsActionButton = [
    {
        xtype: 'spacer'
    },
	{
	    iconCls: 'action',
	    handler: function ()
	    {
            // Remove focus from MEM Number field
            IpPbxMobile.views.settingsCard.query('#memNumber')[0].fieldEl.dom.blur();
            
            // Show Action Sheet
            IpPbxMobile.controllers.MoreController.showSettingsActionSheet();	        
	    }
	}
];


// Reset Action Sheet
var settingsActionSheet = new Ext.ActionSheet(
{
    // Attention: Don't use hideOnMaskTap here!
    modal: true,
    items: [
        {
            text: 'Factory Reset',
            ui: 'decline',
            itemId: 'resetSettingsButtonActionSheet',
            handler: function()
            {
                settingsActionSheet.hide(false);
                IpPbxMobile.controllers.MoreController.doFactoryReset();
            }
        },        
        {
            text: 'Send Log',
            itemId: 'sendLogButtonActionSheet',
            handler: function()
            {
                settingsActionSheet.hide(false);
                logger.sendMobileLog();
            }
        },
        {
            text: 'Cancel',
            itemId: 'cancelButtonActionSheet',
            handler: function()
            {
                settingsActionSheet.hide(false);
            }
        }]
});

//------------------------------------------------------------------------------------------------


