//------------------------------------------------------------------------------------------------
// START VIEW
//------------------------------------------------------------------------------------------------

var startView = null;  // Contains the start page object

//------------------------------------------------------------------------------------------------

IpPbxMobile.views.StartView = Ext.extend(Ext.Panel,
{
    fullscreen: true,
    hidden: true,
    baseCls: 'logo',
    layout:
    {
        type: 'fit',
        pack: 'center',
        align: 'stretch'
    },
    initComponent: function ()
    {
        Ext.apply(this,
		{
		    items: [startViewForm]
		});

        IpPbxMobile.views.StartView.superclass.initComponent.apply(this, arguments);
    }
});

//Register View
Ext.reg('IpPbxMobile.views.StartView', IpPbxMobile.views.StartView);

//------------------------------------------------------------------------------------------------
// BUTTON AND BUTTON PANEL 
//------------------------------------------------------------------------------------------------

// Connect and Login Button
var startViewButton = new Ext.Button(
{
    text: 'Connect',
    ui: 'confirm',
    cls: 'login-button',
    width: 180,
    handler: function ()
    {
        startViewButtonHandler();
    }
});

// Button Container
var startViewButtonPanel = new Ext.Panel(
{
    items: [startViewButton]
});

//------------------------------------------------------------------------------------------------
// URL FIELDSET AND INNER CONTROLS
//------------------------------------------------------------------------------------------------

// URL field
var startViewUrlField = new Ext.form.Text(
{
    name: 'url',
    inputType: 'url',
    autoCapitalize: false,
    placeHolder: 'Server Address',
    inputCls: 'startup-input',
    listeners:
    {
        el:
        {
            touchstart: function ()
            {
                startViewUrlField.setDisabled(false);
            }
        },
        action: function ()
        {
            startViewButtonHandler();
        }
    }
});

// Login Checkbox to enfore a secure connection via HTTPS
var startViewSecurityToggle = new Ext.form.Toggle(
{
    name: 'useSecureConnection',
    labelWidth: '200',
    cls: 'startup-toogle',
    listeners:
    {
        change: function (slider, thumb, newValue, oldValue)
        {
            startViewUrlField[newValue ? 'addCls' : 'removeCls']('lock');
        }
    }
});

// Login Fieldset
var startViewUrlFieldset = new Ext.form.FieldSet(
{
    cls: 'startup-fieldset',
    hidden: true,
    items: [startViewUrlField,
            startViewSecurityToggle]
});

//------------------------------------------------------------------------------------------------
// LOGIN FIELDSET AND INNER CONTROLS
//------------------------------------------------------------------------------------------------

//IpPbx Server ComboBox
var startViewServerBox = new Ext.form.Select(
{
    name: 'serverId',
    hideOnMaskTap: false
});

// Username field
var startViewUsernameField = new Ext.form.Text(
{
    name: 'username',
    autoCapitalize: false,
    autoComplete: false,
    autoCorrect: false,
    placeHolder: 'Username',
    inputCls: 'startup-input',
    listeners:
    {
        el:
        {
            touchstart: function ()
            {
                startViewUsernameField.setDisabled(false);
            }
        },
        beforerender: function (args)
        {
            startViewUsernameField.placeHolder = langData['cn_username'];
        },
        action: function()
        {
            startViewButtonHandler();
        }
    }
});

// Password field
var startViewPasswordField = new Ext.form.Password(
{
    name: 'password',
    autoCapitalize: false,
    autoComplete: false,
    autoCorrect: false,
    placeHolder: 'Password',
    inputCls: 'startup-input',
    listeners:
    {
        el:
        {
            touchstart: function ()
            {
                startViewPasswordField.setDisabled(false);
            }
        },
        beforerender: function (args)
        {
            startViewPasswordField.placeHolder = langData['cn_password'];
        },
        action: function()
        {
            startViewButtonHandler();
        }
    }
});

// Login Fieldset
var startViewLoginFieldset = new Ext.form.FieldSet(
{
    cls: 'startup-fieldset',
    title: '',
    items: [startViewUsernameField,
            startViewPasswordField,
            startViewServerBox]
});

//------------------------------------------------------------------------------------------------

// On Click Handler for Connect / Login button
var startViewButtonHandler = function ()
{
    // Check if Connect button was pressed
    if (startViewButton.getText() === langData['start_btn_connect'])
    {
        // Remove focus from URL field
		startViewUrlField.setDisabled(true); 
        
        Ext.dispatch(
	    {
	        controller: IpPbxMobile.controllers.LoginController,
	        action: 'checkNewUrl',
	        url: startViewUrlField.getValue(),
	        useSecureConnection: startViewSecurityToggle.getValue()
	    });
    }
    else
    {
        // Remove focus from Username and Password field
		startViewPasswordField.fieldEl.dom.blur();
		startViewUsernameField.fieldEl.dom.focus();
        startViewUsernameField.fieldEl.dom.blur();        
		startViewUsernameField.setDisabled(true);
		startViewPasswordField.setDisabled(true);

        setTimeout(function()
		{
			startViewUsernameField.hide(false);
			startViewUsernameField.show();
			startViewPasswordField.hide(false);
			startViewPasswordField.show();
    	},0);
     
        // Hide server selection box
        if (startViewServerBox.picker)
    	{
        	startViewServerBox.picker.onCancelButtonTap();
    	}
        
        // Login button was pressed
        Ext.dispatch(
		{

		    controller: IpPbxMobile.controllers.LoginController,
		    action: 'initLogin',
		    username: startViewUsernameField.getValue(),
		    password: startViewPasswordField.getValue(),
		    serverId: startViewServerBox.getValue()
		});
    }
}

//------------------------------------------------------------------------------------------------
// FORM PANEL
//------------------------------------------------------------------------------------------------

// Start View Form
var startViewForm = new Ext.form.FormPanel(
{
    fullscreen: true,
    componentCls: 'startup-form',
    hidden: false,
    items: [
    startViewUrlFieldset,
    startViewLoginFieldset,
    startViewButtonPanel]
});

//------------------------------------------------------------------------------------------------



