﻿//------------------------------------------------------------------------------------------------
// USER PROFILE VIEW
//------------------------------------------------------------------------------------------------

var userStatusTextUpdateTask = null;

IpPbxMobile.views.UserProfileView = Ext.extend(Ext.Panel,
{
	title: 'Profile',
    id: 'userprofileview',
	scroll: 'vertical',
	styleHtmlContent: false,
    layout:
    {
		type: "vbox",
		align: "stretch"
	},
	
	initComponent: function()
    {
        var profileToolbar = new Ext.Toolbar(
        {
	        dock: 'top',
	        title: 'My Profile',
            itemId: 'profileToolbar',
            items: [
            {
	            xtype: 'spacer'
	        },
            {
                xtype: 'button',
                iconMask: true,
                ui: 'plain',
                iconCls: 'action',
	            handler: function()
                {
                    // Remove focus from status text field
                    profileUserStatusText.fieldEl.dom.blur();
                    profileUserStatusText.setDisabled(true);

                    // Show Action Sheet
                    profileActionSheet.show();
                }
            }]
        });

		Ext.apply(this,
        {
			dockedItems : [ profileToolbar ],
			items : [
            {
                xtype  : 'panel', 
				margin : '8 8 0 8',
				height : 110,
				layout :
                {
					type : "hbox",
					align : "stretch"
				},
				items : [ profileUserPicture, profileUserInfoPanel ]
			},
            {
                xtype  : 'panel',
                 
				margin : '8 8 0 8',
				height : 55,
				layout : {
					type : "hbox",
					align : "stretch"
				},
				items : [ profileUserStatusIcon, profileUserStatusFieldSet ]
			},
            {
                xtype  : 'fieldset', 
            	margin : '8 8 8 8',
				cls: 'ux-layout-fit',

				items : [ profileUserStatusText ]
			}]            
		});

		IpPbxMobile.views.UserProfileView.superclass.initComponent.apply(this,arguments);
	},

    listeners:
    {
        afterrender: function(cmp)
        {
            // Create delayed task for status text update
            userStatusTextUpdateTask = new Ext.util.DelayedTask(function()
            {
                IpPbxMobile.controllers.UserController.updateUserFreeText(
                    { statusText: profileUserStatusText.getValue() } );
            });

            // Register tap event for main element to submit status text
            // when tapping on panel
            cmp.el.on('tap', function()
            {
                userStatusTextUpdateTask.delay(200);
            });
        }
    }
});

// Register View
Ext.reg('IpPbxMobile.views.UserProfileView', IpPbxMobile.views.UserProfileView);

// ------------------------------------------------------------------------------------------------

// Action Sheet for User Switching
var profileActionSheet = new Ext.ActionSheet(
{
    //hideOnMaskTap: true, // Attention: This causes a dead application.
    modal: true,
    id: 'profileActionSheet',
    items: [
    {
        text: 'Switch user',
        ui  : 'decline',
        itemId: 'switchUserButtonActionSheet',
        handler: function()
        {
            profileActionSheet.hide(false);
            IpPbxMobile.controllers.UserController.doUserSwitch();                        
        }
    },
    {
        text: 'New User Picture',
        // TODO: Remove the following line to activate the "User Picture Capture" feature
        hidden: true,
        itemId: 'captureButtonActionSheet',
        handler: function()
        {
            profileActionSheet.hide(false);
            IpPbxMobile.controllers.UserController.captureUserPicture();
            profileUserStatusText.setDisabled(false);
        }
    },
    {
        text: 'Cancel',
        itemId: 'cancelButtonActionSheet',
        handler: function()
        {            
        	profileActionSheet.hide(false);
            profileUserStatusText.setDisabled(false);
        }
    }]
});

// ------------------------------------------------------------------------------------------------

var profileUserPicture = new Ext.form.FieldSet(
{
	margin: '0',
    listeners: 
    {
        el:
        {
            tap: function()
            {   
                // TODO: Uncomment the following line to activate the "User Picture Capture" feature
                //IpPbxMobile.controllers.UserController.captureUserPicture();
            }
        }
    }
});

var profileUserStatusIcon = new Ext.Panel(
{
	margin : '0',
    listeners : 
    {
        afterrender: function(ele)
        {
            ele.el.on('click', function()
            {
                IpPbxMobile.controllers.UserController.showUserStateChangePage();
            });
        }        
    }
});

var profileUserName = new Ext.Panel(
{
	cls : 'profileTextNormal',
    margin : '0 0 4 0',
});

var profileForwardingState = new Ext.Panel(
{
	cls : 'profileTextSmaller'
});

var profileUserInfoPanel = new Ext.Panel(
{
	margin : '4 0 0 8',
    items : [profileUserName, profileForwardingState]
});

// ------------------------------------------------------------------------------------------------

var profileUserStatus = new Ext.Container(
{	
    cls:'arrow',
    baseCls:'x-field',
    padding: '12 8 12 8',
    style: 'background-color: white',
	listeners:
    {
		el: 
        {
           touchend: function()
            {
               IpPbxMobile.controllers.UserController.showUserStateChangePage();
            }
		}
	}
});

var profileUserStatusFieldSet = new Ext.form.FieldSet(
{
	style: '-webkit-box-flex: 1;',
	margin: '0 0 0 8',
	items: [ profileUserStatus ]
});

// ------------------------------------------------------------------------------------------------

var profileUserStatusText = new Ext.ux.AutofitTextArea(
{	
    maxlength: 256,	
	name: 'statetext',
    itemId: 'statetext',
	value: 'Your StatusText',
	autoComplete: false,
	autoCapitalize: false,
	autoCorrect: false,
    allowEnter: false,
    cls: 'profileUserStatusText',
    listeners:
    {
		change: function(field, newValue, oldValue)
        {          
            userStatusTextUpdateTask.delay(200);
		},
        blur: function(cmp)
        {   
            userStatusTextUpdateTask.delay(200);
        }
	}
});

// ------------------------------------------------------------------------------------------------
