﻿//------------------------------------------------------------------------------------------------
// MAIN VIEWPORT
//------------------------------------------------------------------------------------------------

IpPbxMobile.views.Viewport = Ext.extend(Ext.TabPanel,
{
    id: 'viewport',
    fullscreen: true,
    innerHeight: 0,
    cardSwitchAnimation: false, // Bugfix for 6793 and 6884 (PD 17.10.2011) 

    initComponent: function()
    {
        // Create main views
        Ext.apply(IpPbxMobile.views,
    	{
    	    userCard: new IpPbxMobile.views.UserCard(),
    	    contactsCard: new IpPbxMobile.views.ContactsCard(),
    	    journalCard: new IpPbxMobile.views.JournalCard(),
    	    dialpadCard: new IpPbxMobile.views.DialpadCard(),
    	    moreCard: new IpPbxMobile.views.MoreCard()
    	});

        // Translate Main GUI
        IpPbxMobile.controllers.LanguageController.translateMainGui();

        // Add views to viewport
        Ext.apply(this,
		{		    
            tabBar:
			{
			    dock: 'bottom',
			    layout:
				{
				    pack: 'center'
				}
			},
		    
		    items: [
                IpPbxMobile.views.userCard,
                IpPbxMobile.views.contactsCard,
                IpPbxMobile.views.journalCard,
                IpPbxMobile.views.dialpadCard,
                IpPbxMobile.views.moreCard
	        ]
		});

        IpPbxMobile.views.Viewport.superclass.initComponent.apply(this, arguments);
    },

    getCurrentCardId: function()
	{
		var currCard = this.getActiveItem();
		if (currCard)
		{
		    return currCard.getId();
		}

		return "";
	},

    onCardSwitch: function(newCard, oldCard, newCardIndex)
    {
        // Validate card object
        if (newCard == null)
            return;

        // Get card id
        var newCardId = newCard.getId();

        // Necessary for iPhone: Resize the view if a second
        // statusbar appears e.g. incoming call, hotspot...
        if (this.innerHeight !== document.body.clientHeight &&
            newCardId !== 'dialpadcard')
        {
            logger.log("Resize View - Height: " + document.body.clientHeight);
            this.innerHeight = document.body.clientHeight;
            Ext.EventManager.orientationEvent.fire(Ext.orientation, document.body.clientWidth, document.body.clientHeight);
        }

        // Check if something must be done for
        // the card which will be displayed
        switch (newCardId)
        {
            case 'usercard':                
                // Init User Profile Page Update                
                logger.log('Updating My Profile');

                var nextValidUpdateTS = lastProfileChange + 15;

                if (getUnixTimestamp() < nextValidUpdateTS)
                {
                    logger.log('Updating My Profile - Aborted because of 15 seconds wait mechanism.');
                }
                else
                {
                    var profileTask = new Ext.util.DelayedTask(function ()
                    {
                        IpPbxMobile.controllers.UserController.getPresenceData(true);
                    });

                    profileTask.delay(100);
                }
								
                // Force to update height of statustextbox. 
                // It prevents a wrong height in case of card switch.
                // Warning: Please don't delete!
                profileUserStatusText.updateHeight();

                break;

            case 'contactscard':
                // Enable state update timer task
                IpPbxMobile.controllers.RemoteContactController.enableUpdateStateTask(true);
                break;

            case 'journalcard':
                // Init Call Journal Update
                logger.log('Updating Call journal');

                var journalTask = new Ext.util.DelayedTask(function ()
                {
                    IpPbxMobile.controllers.JournalController.loadJournal(true);
                });

                journalTask.delay(100);
                break;

            case 'dialpadcard':
                logger.log('Cleaning Dialpad Number Field');

                // Reset redial button
                IpPbxMobile.stores.JournalRedialStore.clearData();

                // Clear number field
                dialpadNumberField.setText('');
                break;
        }
        
        // If this is NOT the ContactsCard ensure
        // that UpdateStateTask is disabled 
        if (newCardId !== 'contactscard')
        {
            // Disable contact state update timer
            IpPbxMobile.controllers.RemoteContactController.enableUpdateStateTask(false);
        }

        // If we are switching to another card than usercard,
        // ensure we are updating the status text (on some devices
        // the blur event is not fired properly)
        if (newCardId !== 'usercard')
        {
            // Remove focus from status text field
            profileUserStatusText.fieldEl.dom.blur();
        }

        // Check if we need to remove back button event listener.
        IpPbxMobile.checkBackButtonHandler(newCardId);
    }

});
