//-------------------------------------------------------------------------
// IPPBX MOBILE - SENCHA TOUCH MODS
//-------------------------------------------------------------------------

// This file contains the following types of modifications:
//
// 1. OBJECT OVERRIDES
//    Note: This is the dark side of coding
//    Use this way only if there are no other possibilities.
//    Otherwise you will go directly to the JavaScript hell :-)
//
// 2. OBJECT EXTENSIONS
//    Here we add methods to existing controls / classes
//
// 3. NEW INHERITED OBJECTS
//    Extend existing control with the required functionality.
//    This is the best way; the light side but still JavaScript :-(

//-------------------------------------------------------------------------
// 1. OBJECT OVERRIDES - !!! DIRTY !!!
//-------------------------------------------------------------------------

// Problem:
// SelectField (Comboboxes / Ext.Picker) does
// not work if animations are disabled.
Ext.override(Ext.Picker,
{
    onDoneButtonTap: function ()
    {
        this.hide(false);
        this.fireEvent('change', this, this.getValue());
    },

    onCancelButtonTap: function ()
    {
        this.hide(false);
        this.setValue(this.values);
        this.fireEvent('cancel', this);
    }
});

//-------------------------------------------------------------------------
// 2. OBJECT EXTENSIONS
//-------------------------------------------------------------------------

// Add new method "SetLabel" for text fields.
// This makes the localisation process much easier.
Ext.override(Ext.form.Field,
{
    setLabel: function (text)
    {
        if (this.rendered)
        {
            this.labelEl.dom.innerHTML = text;
        }
        this.label = text;
    }
});

//-------------------------------------------------------------------------

// Problem:  On some DetailCards (Remote and Journal cards) scrolling is not possible
// Reason:   The component size is not calculated correctly
// Solution: We add a new method to enforce the correct size calculation
Ext.override(Ext.lib.Component,
{
    updateWithLastComponentSize: function (htmlOrData, loadScripts, cb)
    {
        if (this.tpl && !Ext.isString(htmlOrData))
        {
            this.data = htmlOrData;
            if (this.rendered)
            {
                this.tpl[this.tplWriteMode](this.getTargetEl(), htmlOrData || {});
            }
        }
        else
        {
            this.html = Ext.isObject(htmlOrData) ? Ext.DomHelper.markup(htmlOrData) : htmlOrData;
            if (this.rendered)
            {
                this.getTargetEl().update(this.html, loadScripts, cb);
            }
        }
        if (this.rendered)
        {
            this.doComponentLayout(this.componentLayout.lastComponentSize.width, this.componentLayout.lastComponentSize.height, false);
        }
    }
});

//-------------------------------------------------------------------------
// 3. NEW INHERITED OBJECTS
//-------------------------------------------------------------------------

// Selfmade improved version of the Ext.is object
Ext.ux.is = {

    Android4: false,
    
    init: function()
    {   
        // Get correct navigator object            
        navigator = navigator || window.navigator;
         
        // Get values from Ext.is object
        var platforms = Ext.is.platforms,
            ln = platforms.length,
            i, platform;
        
        for (i = 0; i < ln; i++)
        {
            platform = platforms[i];
            this[platform.identity] = platform.regex.test(navigator[platform.property]);
        }
        
        // Add own special values
        platforms = this.platforms;
        ln = platforms.length;            
            
        for (i = 0; i < ln; i++)
        {
            platform = platforms[i];
            this[platform.identity] = platform.regex.test(navigator[platform.property]);
        }
    },

    platforms: [
    {
        property: 'userAgent',
        regex: /Android 4/i,
        identity: 'Android4'
    }]
};

Ext.ux.is.init();

//-------------------------------------------------------------------------

Ext.ux.DocumentHelper = Ext.extend(Object,
{
    constructor: function (config)
    {
        Ext.apply(this, config);
    },

    // Check if HD-display and change density.
    setRightDensity: function ()
    {
        if (Ext.is.Android && document.width > 640)
        {
            var head = document.getElementsByTagName('head')[0];
            var tag = document.createElement('meta');
            tag.setAttribute('name', 'viewport');
            tag.setAttribute('content', 'target-densitydpi = 160');
            head.appendChild(tag);
        }
    }
});

//-------------------------------------------------------------------------

Ext.ux.AdvancedSearch = Ext.extend(Ext.form.Search,
{
    showSearchButton: true,
    fieldCls: 'ux-input-advancedsearch',
    renderTpl: [
        '<div class="ux-field-advancedsearch">',
        '<tpl if="label">',
            '<div class="x-form-label"><span>{label}</span></div>',
        '</tpl>',
        '<tpl if="fieldEl">',
            '<div class="x-form-field-container x-input-search">',
                '<tpl if="showSearchButton"><form role="search" onsubmit="return false"></tpl>',
                    '<input id="{inputId}" type="{inputType}" name="{name}" class="{fieldCls}"',
                        '<tpl if="tabIndex">tabIndex="{tabIndex}" </tpl>',
                        '<tpl if="placeHolder">placeholder="{placeHolder}" </tpl>',
                        '<tpl if="style">style="{style}" </tpl>',
                        '<tpl if="minValue != undefined">min="{minValue}" </tpl>',
                        '<tpl if="maxValue != undefined">max="{maxValue}" </tpl>',
                        '<tpl if="stepValue != undefined">step="{stepValue}" </tpl>',
                        '<tpl if="autoComplete">autocomplete="{autoComplete}" </tpl>',
                        '<tpl if="autoCapitalize">autocapitalize="{autoCapitalize}" </tpl>',
                        '<tpl if="autoFocus">autofocus="{autoFocus}" </tpl>',
                    '/>',
                '<tpl if="showSearchButton"></form></tpl>',
                '<tpl if="useMask"><div class="x-field-mask"></div></tpl>',
            '</div>',
        '</tpl>',
        '<tpl if="useClearIcon">',
            '<div class="x-field-clear-container">',
                '<div class="x-field-clear x-hidden-visibility">&#215;</div>',
            '</div>',
        '</tpl>',
        '</div>'
    ],

    onRender: function ()
    {
        this.renderData.showSearchButton = this.showSearchButton;
        this.renderData.fieldCls = this.fieldCls;
        Ext.ux.AdvancedSearch.superclass.onRender.apply(this, arguments);
    }
});

Ext.reg('advancedsearchfield', Ext.ux.AdvancedSearch);

//-------------------------------------------------------------------------

Ext.ux.AutofitTextArea = Ext.extend(Ext.form.TextArea,
{
    maxRows: undefined,
    initialHeight: undefined,
    allowEnter: true,

    renderTpl: [
        '<tpl if="label">',
            '<div class="x-form-label"><span>{label}</span></div>',
        '</tpl>',
        '<tpl if="fieldEl">',
            '<div class="x-form-field-container">',
                '<textarea id="{inputId}" type="{type}" name="{name}" class="{fieldCls}"',
                    '<tpl if="tabIndex">tabIndex="{tabIndex}" </tpl>',
                    '<tpl if="placeHolder">placeholder="{placeHolder}" </tpl>',
                    '<tpl if="style">style="{style}" </tpl>',
                    '<tpl if="maxRows != undefined">rows="{maxRows}" </tpl>',
                    '<tpl if="maxlength">maxlength="{maxlength}" </tpl>',
                    '<tpl if="autoComplete">autocomplete="{autoComplete}" </tpl>',
                    '<tpl if="autoCapitalize">autocapitalize="{autoCapitalize}" </tpl>',
                    '<tpl if="autoFocus">autofocus="{autoFocus}" </tpl>',
                '></textarea>',
                '<tpl if="useMask"><div class="x-field-mask"></div></tpl>',
            '</div>',
        '</tpl>',
        '<textarea class="clone" style="position:absolute; visibility:hidden;"></textarea>'
    ],

    initComponent: function()
    {
        Ext.ux.AutofitTextArea.superclass.initComponent.apply(this, arguments);

        this.addEvents('keydown');
    },

    initEvents: function()
    {
        Ext.ux.AutofitTextArea.superclass.initEvents.apply(this, arguments);

        if (this.fieldEl)
        {
            this.mon(this.fieldEl, {
                keydown: this.onKeyDown,
                scope: this
            });
        }
    },

    updateHeight: function()
    {
        //Use clone textarea to determine the necessary height
        if (this.el != undefined)
        {
            var clone = this.el.down('.clone');
            clone.dom.innerHTML = this.getValue();
            //Scale textarea if content doesn't fit to it           
            this.setHeight(clone.dom.scrollHeight);
        }
    },

    afterRender: function()
    {
        Ext.ux.AutofitTextArea.superclass.afterRender.apply(this, arguments);
        this.updateHeight();
    },

    setValue: function(value)
    {
        Ext.ux.AutofitTextArea.superclass.setValue.apply(this, arguments);
        this.updateHeight();
    },

    onKeyUp: function(e)
    {
        Ext.ux.AutofitTextArea.superclass.onKeyUp.apply(this, arguments);
        this.updateHeight();
    },

    onKeyDown: function(e)
    {
        //Suppress enter event, a line break is not allowed
        if (!this.allowEnter && e.browserEvent.keyIdentifier === 'Enter')
        {
            e.browserEvent.returnValue = false;
        }
    },

    updateClearIconVisibility: function(e)
    {
        Ext.ux.AutofitTextArea.superclass.updateClearIconVisibility.apply(this, arguments);        
        this.updateHeight();
    },

    onRender: function()
    {
        this.renderData.maxRows = this.maxRows;
        this.renderData.maxlength = this.maxlength;
        Ext.ux.AutofitTextArea.superclass.onRender.apply(this, arguments);
    }
});

Ext.reg('autofittextarea', Ext.ux.AutofitTextArea);

//-------------------------------------------------------------------------

// Needs the following CSS classes:
// ux-result-mask
// ux-result-mask-msg
// ux-result-mask-img
// ux-result-mask-info
// ux-result-mask-error
// ux-result-mask-success

Ext.ux.ResultMask = Ext.extend(Ext.util.Observable,
{
    msg: '',
    type: 'info',
    timeout: 3000,

    callback: null,
    redirect: null,

    autoShow: true,
    hidden: true,

    imageCls: '',
    msgCls: 'ux-result-mask',

    resultMaskTask: null,

    constructor: function (el, config)
    {
        // Get element from DOM
        this.el = Ext.get(el);

        // Apply config
        Ext.apply(this, config);

        // Run base constructor
        Ext.ux.ResultMask.superclass.constructor.call(this);

        // Check if mask must be shown automatically after construction
        if (this.autoShow)
        {
            this.show();
        }
    },

    cancelTask: function ()
    {
        // Cancel task (if present)
        if (this.resultMaskTask)
        {
            // Stop task
            this.resultMaskTask.cancel();
            this.resultMaskTask = null;
        }
    },

    isVisible: function ()
    {
        return !this.hidden;
    },

    show: function ()
    {
        // Check if mask is visible already
        if (this.isVisible())
            return;

        // Set mask image
        switch (this.type)
        {
            case 'ok':
            case 'success':
                this.imageCls = 'ux-result-mask-success';
                break;
            case 'error':
                this.imageCls = 'ux-result-mask-error';
                break;
            default: // Used for info image
                this.imageCls = 'ux-result-mask-info';
                break;
        }

        // Create event handler
        this.el.on(
            {
                tap: this.hide,
                scope: this
            });

        // Show mask
        this.el.mask('<div class="ux-result-mask-img ' + this.imageCls + '">&nbsp;</div><div class="ux-result-mask-msg">' + this.msg + '</div>', this.msgCls, false);

        // Modify hidden flag
        this.hidden = false;

        // Check for mask timer
        if (this.timeout && this.timeout > 0)
        {
            this.resultMaskTask = new Ext.util.DelayedTask(this.hide, this);
            this.resultMaskTask.delay(this.timeout);
        }
    },

    hide: function ()
    {
        // Check if mask is visible already
        if (!this.isVisible())
            return;

        // Disable mask timer
        this.cancelTask();

        // Remove event handler
        this.el.un('tap');

        // Hide mask
        this.el.unmask();

        // Check for callback function
        if (this.callback && (typeof this.callback == 'function'))
        {
            this.callback.call();
        }
        else if (this.redirect)
        {
            Ext.dispatch(
			{
			    controller: this.redirect.controller,
			    action: this.redirect.action
			});
        }

        // Modify hidden flag
        this.hidden = true;

        // Remove all listeners
        this.clearListeners();
    }

});

//-------------------------------------------------------------------------

// ADDON: Improved JSONP Class
//Adds error handling and makes every WebCall unique.
//Therefore an additional parameter is added with the WebCallId.
//This prevents browser result caching.

//Ext.ux.WebCallCounter=0;

//Ext.util.JSONP =
//{
//	queue: [],
//	current: null,
//	useBrowserCache: false,
//	source: null,
//	
//	request: function(o)
//	{
//		o = o || {};
//		if (!o.url)
//		{
//			return;
//		}

//		// PhoneGap Support
//		if (navigator.network !== undefined && navigator.network.connection.type === Connection.NONE)
//		{    	
//			this.onRequestFailure(null, 10);
//		}
//		
//		var me = this;
//		o.params = o.params || {};
//     
//		if (o.callbackKey !== undefined)
//			o.params[o.callbackKey] = 'Ext.util.JSONP.callback';
//		else o.params['callback'] = 'Ext.util.JSONP.callback';
//     	
//		if (!this.useBrowserCache)
//		{
//			Ext.ux.WebCallCounter++;
//			o.params['uxx'] = Ext.ux.WebCallCounter;
//		}
//		
//		var params = Ext.urlEncode(o.params);

//		var script = document.createElement('script');
//		script.type = 'text/javascript';
//     
//		Ext.fly(script).on(
//		{
//			error: Ext.util.Functions.createDelegate(this.onRequestTimeout, this)
//		});

//		this.queue.push(
//		{
//			url: o.url,
//			script: script,
//			callback: o.callback || function(){},
//			onFailure: o.onFailure || function(){},
//			scope: o.scope || window,
//			params: params || null,
//			onFailureHandler: o.onFailureHandler
//		});

//		if (!this.current)
//		{
//			this.next();
//		}
//	},

//	next: function()
//	{
//		this.current = null;
//		if (this.queue.length)
//		{
//			this.current = this.queue.shift();
//			this.current.script.src = this.current.url + (this.current.params ? ('?' + this.current.params) : '');
//			document.getElementsByTagName('head')[0].appendChild(this.current.script);
//		}
//	},
//	
//	callback: function(json, httpCode)
//	{
//		if (httpCode === undefined)
//    	{
//			httpCode = 200;
//    	}
//		
//		if (json === null && httpCode === 200)
//		{
//			httpCode = 204;
//		}
//		
//		if (httpCode === 200)
//		{
//			this.current.callback.call(this.current.scope, json);
//			document.getElementsByTagName('head')[0].removeChild(this.current.script);
//			this.next();			
//		}
//		else
//		{
//			this.onRequestFailure(json, httpCode);
//		}
//	},
//	
//	onRequestFailure: function(json, httpCode)
//	{
//		if (this.current.onFailureHandler !== undefined)
//		{
//			Ext.dispatch(
//			{
//			    controller: this.current.onFailureHandler.failureController,
//			    action: 'requestFailed',
//			    sourceController: this.current.onFailureHandler.sourceController,
//			    sourceAction: this.current.onFailureHandler.sourceAction,
//			    isBackgroundProcess: this.current.onFailureHandler.isBackgroundProcess,
//			    httpCode: httpCode
//			});
//		}
//		else
//		{
//			this.current.onFailure.call(this.current.scope, json, httpCode);
//		}
//		
//		document.getElementsByTagName('head')[0].removeChild(this.current.script);
//		this.next();
//	},
//	
//	onRequestTimeout: function(browserEvent)
//	{
//		if (this.current.onFailureHandler !== undefined)
//		{
//			Ext.dispatch(
//			{
//			    controller: this.current.onFailureHandler.failureController,
//			    action: 'requestFailed',
//			    sourceController: this.current.onFailureHandler.sourceController,
//			    sourceAction: this.current.onFailureHandler.sourceAction,
//			    isBackgroundProcess: this.current.onFailureHandler.isBackgroundProcess,
//			    httpCode: 118 // Connection timed out
//			});
//		}
//		else
//		{
//			this.current.onFailure.call(this.current.scope, browserEvent, 118);
//		}
//		
//		document.getElementsByTagName('head')[0].removeChild(this.current.script);
//		this.next();
//	}
//};

//-------------------------------------------------------------------------

